#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module is part of the Python Pool library. It defines the base classes
for"""

__all__ = ["PoolController", "PoolPseudoMotorController",
           "PoolPseudoCounterController"]

__docformat__ = 'restructuredtext'

import sys
import weakref
import StringIO
import traceback
import functools

from taurus.core.util import CaselessDict

from sardana import State, ElementType, TYPE_TIMERABLE_ELEMENTS
from sardana.sardanaevent import EventType
from sardana.sardanavalue import SardanaValue

from .poolextension import translate_ctrl_value
from .poolbaseelement import PoolBaseElement


class PoolBaseController(PoolBaseElement):
    """Base class for all controllers"""
    def __init__(self, **kwargs):
        self._ctrl = None
        self._ctrl_error = None
        self._element_ids = {}
        self._pending_element_ids = {}
        self._element_axis = {}
        self._pending_element_axis = {}
        self._element_names = CaselessDict()
        self._pending_element_names = CaselessDict()
        self._operator = None
        kwargs['elem_type'] = ElementType.Controller
        super(PoolBaseController, self).__init__(**kwargs)
    
    def get_ctrl_types(self):
        raise NotImplementedError
    
    def get_ctrl_type_names(self):
        return map(ElementType.whatis, self.get_ctrl_types())
        
    def is_online(self):
        return True
    
    def get_ctrl_error(self):
        return self._ctrl_error
    
    def get_ctrl_error_str(self):
        """"""
        err = self._ctrl_error
        if err is None:
            return ""
        sio = StringIO.StringIO()
        traceback.print_exception(err[0], err[1], err[2], None, sio)
        s = sio.getvalue()
        sio.close()
        if s[-1:] == "\n":
            s = s[:-1]
        return s
    
    def add_element(self, elem, propagate=1):
        name, axis, eid = elem.get_name(), elem.get_axis(), elem.get_id()
        if self.is_online():
            try:
                self._ctrl.AddDevice(axis)
            except:
                self.error("Unable to add %s(%s)", name, axis, exc_info=1)
                self._pending_element_ids[eid] = elem
                self._pending_element_axis[axis] = elem
                self._pending_element_names[name] = elem
            self._element_ids[eid] = elem
            self._element_axis[axis] = elem
            self._element_names[name] = elem
        else:
            #TODO: raise exception
            self._pending_element_ids[eid] = elem
            self._pending_element_axis[axis] = elem
            self._pending_element_names[name] = elem
        if propagate:
            elements = self.get_elements()
            elements = [ elements[_id].name for _id in sorted(elements) ]
            self.fire_event(EventType("elementlist", priority=propagate),
                            elements)
    
    def remove_element(self, elem, propagate=1):
        name, axis, eid = elem.get_name(), elem.get_axis(), elem.get_id()
        f = self._element_ids.has_key(eid)
        if not f:
            f = self._pending_element_ids.has_key(eid)
            if not f:
                raise Exception("element '%s' is not in controller")
            del self._pending_element_ids[eid]
            del self._pending_element_axis[axis]
            del self._pending_element_names[name]
        else:
            del self._element_ids[eid]
            del self._element_axis[axis]
            del self._element_names[name]
            try:
                self._ctrl.DeleteDevice(axis)
            except:
                self.error("Unable to delete %s(%s)", name, axis, exc_info=1)
        if propagate:
            elements = self.get_elements()
            elements = [ elements[_id].name for _id in sorted(elements) ]
            self.fire_event(EventType("elementlist", priority=propagate),
                            elements)

    def remove_axis(self, axis, propagate=1):
        f = self._element_axis.has_key(axis)
        if not f:
            f = self._pending_element_axis.has_key(axis)
            if not f:
                raise Exception("element '%s' is not in controller")
            elem = self._pending_element_axis[axis]
        else:
            elem = self._element_axis[axis]
        self.remove_element(elem, propagate=propagate)
        
    def get_elements(self):
        return self._element_ids
    
    def get_element_ids(self):
        return self._element_ids
    
    def get_element_axis(self):
        return self._element_axis
    
    def get_element(self, **kwargs):
        k = kwargs.get('axis')
        if k is None:
            k = kwargs.get('name')
            if k is None:
                k = kwargs.get('id')
                if k is None:
                    raise Exception("Must give either name, id or axis")
                d, pd = self._element_ids, self._pending_element_ids
            else:
                d, pd = self._element_names, self._pending_element_names
        else:
            d, pd = self._element_axis, self._pending_element_axis
        
        elem = d.get(k)
        if elem is None:
            elem = pd.get(k)
        return elem

    def read_axis_states(self, axes=None):
        """Reads the state for the given axes. If axes is None, reads the
        state of all active axes.
        
        :param axes: the list of axis to get the state. Default is None meaning
                       all active axis in this controller
        :type axes: seq<int> or None
        :return: a map containing the controller state information for each axis
        :rtype: dict<PoolElement, state info>
        """
        raise NotImplementedError
    
    def read_axis_values(self, axes=None):
        """Reads the value for the given axes. If axes is None, reads the
        value of all active axes.
        
        :param axes: the list of axis to get the value. Default is None meaning
                       all active axis in this controller
        :type axes: seq<int> or None
        :return: a map containing the controller value information for each axis
        :rtype: dict<PoolElement, value>
        """
        raise NotImplementedError


def check_ctrl(fn):
    @functools.wraps(fn)
    def wrapper(pool_ctrl, *args, **kwargs):
        if not pool_ctrl.is_online():
            raise Exception("Cannot execute '%s' because '%s' is offline" % \
                            (fn.__name__, pool_ctrl.name))
        return fn(pool_ctrl, *args, **kwargs)
    return wrapper

def ctrl_access(fn):
    @functools.wraps(fn)
    def wrapper(pool_ctrl, *args, **kwargs):
        with pool_ctrl:
            return fn(pool_ctrl, *args, **kwargs)
    return wrapper


class PoolController(PoolBaseController):
    """Controller class mediator for sardana controller plugins"""
    
    def __init__(self, **kwargs):
        self._lib_info = kwargs.pop('lib_info')
        self._ctrl_info = kwargs.pop('class_info')
        self._lib_name = kwargs.pop('library')
        self._class_name = kwargs.pop('klass')
        self._properties = kwargs.pop('properties')
        super(PoolController, self).__init__(**kwargs)
        self.re_init()

    def serialize(self, *args, **kwargs):
        kwargs = PoolBaseController.serialize(self, *args, **kwargs)
        ctrl_info = self._ctrl_info
        if ctrl_info is None:
            kwargs['module'] = self._lib_name
            kwargs['klass'] = self._class_name
            kwargs['language'] = 'Python'
            kwargs['file_name'] = None
            kwargs['types'] = None
            kwargs['main_type'] = None
            kwargs['parent'] = self._class_name
        else:
            types = self.get_ctrl_type_names()
            kwargs['module'] = ctrl_info.module_name
            kwargs['klass'] = ctrl_info.name
            kwargs['language'] = 'Python'
            kwargs['file_name'] = ctrl_info.file_name
            kwargs['types'] = types
            kwargs['parent'] = ctrl_info.name
            if len(types):
                kwargs['main_type'] = types[0]
            else:
                kwargs['main_type'] = None
        return kwargs
    
    def _create_ctrl_args(self):
        name = self.name
        klass = self._ctrl_info.klass
        props = dict(self._properties)
        args, kwargs = [], dict(pool_controller=weakref.ref(self))
        return name, klass, props, args, kwargs
    
    def _create_controller(self):
        name, klass, props, args, kwargs = self._create_ctrl_args()
        api = self._ctrl_info.api_version
        if api == 0:
            ctrl = klass(name, props)
            ctrl._args = args
            ctrl._kwargs = kwargs
        elif api == 1:
            ctrl = klass(name, props, *args, **kwargs)
        return ctrl

    def _init(self):
        if self._ctrl_info is None:
            if self._lib_info is not None:
                self._ctrl_error = self._lib_info.get_error()
            return
        try:
            self._ctrl = self._create_controller()
        except:
            self._ctrl = None
            self._ctrl_error = sys.exc_info()
    
    def re_init(self):
        self.set_state(State.Init, propagate=2)
        status = "{0} is Initializing (temporarily unavailable)".format(self.name)
        self.set_status(status, propagate=2)
        manager = self.pool.ctrl_manager
        old_e_ids = self._element_ids
        old_p_e_ids = self._pending_element_ids
        
        elem_axis = dict(self._element_axis)
        for axis in elem_axis:
            self.remove_axis(axis, propagate=0)
        
        if self._lib_info is None:
            mod_name = self.get_library_name()
        else:
            mod_name = self._lib_info.name

        if self._ctrl_info is None:
            class_name = self.get_class_name()
        else:
            class_name = self._ctrl_info.name
        
        self._ctrl_error = None
        self._ctrl_info = None
        self._lib_info = manager.getControllerLib(mod_name)
        if self._lib_info is not None:
            self._ctrl_info = self._lib_info.get_controller(class_name)
        self._init()
        
        for elem in elem_axis.values():
            self.add_element(elem, propagate=0)
    
        state, status = State.Fault, ""
        if self.is_online():
            state = State.On
        else:
            status = "\n" + self.get_ctrl_error_str()
        
        status = "{0} is {1}".format(self.name, State[state]) + status
        self.set_status(status, propagate=2)
        self.set_state(state, propagate=2)

    def get_ctrl_types(self):
        return self._ctrl_info.types

    def is_timerable(self):
        for t in self._ctrl_info.types:
            if t in TYPE_TIMERABLE_ELEMENTS:
                return True
        return False

    def is_online(self):
        return self._ctrl_error is None and self._ctrl is not None
    
    def get_ctrl(self):
        return self._ctrl
    
    ctrl = property(fget=get_ctrl, doc="actual controller object")
    
    def get_ctrl_info(self):
        return self._ctrl_info
    
    ctrl_info = property(fget=get_ctrl_info, doc="controller information object")
    
    def set_operator(self, operator):
        """Defines the current operator object for this controller.
           For example, in acquisition, it should be a :class:`PoolMeasurementGroup`
           object.
           
           :param operator: the new operator object
           :type operator: object"""
        self._operator = operator
    
    def get_operator(self):
        return self._operator
    
    operator = property(fget=get_operator, fset=set_operator, doc="current controller operator")
    
    # START API WHICH ACCESSES CONTROLLER API ----------------------------------
    
    @check_ctrl
    def set_log_level(self, level):
        self.ctrl._log.log_obj.setLevel(level)
    
    @check_ctrl
    def get_log_level(self):
        return self.ctrl._log.log_obj.level
    
    def get_library_name(self):
        return self._lib_name
    
    def get_class_name(self):
        return self._class_name
    
    @check_ctrl
    def get_axis_attributes(self, axis):
        return self.ctrl.GetAxisAttributes(axis)
    
    @check_ctrl
    def get_ctrl_attr(self, name):
        ctrl_info = self.ctrl_info
        attr_info = ctrl_info.ctrl_attributes[name]
        if hasattr(self.ctrl, attr_info.fget):
            return getattr(self.ctrl, attr_info.fget)()
        else:
            return self.ctrl.GetCtrlPar(name)
    
    @check_ctrl
    def set_ctrl_attr(self, name, value):
        ctrl_info = self.ctrl_info
        attr_info = ctrl_info.ctrl_attributes[name]
        if hasattr(self.ctrl, attr_info.fset):
            return getattr(self.ctrl, attr_info.fset)(value)
        else:
            return self.ctrl.SetCtrlPar(name, value)
    
    @check_ctrl
    def get_axis_attr(self, axis, name):
        ctrl_info = self.ctrl_info
        axis_attr_info = ctrl_info.axis_attributes[name]
        if hasattr(self.ctrl, axis_attr_info.fget):
            ret = getattr(self.ctrl, axis_attr_info.fget)(axis)
        else:
            ret = self.ctrl.GetAxisExtraPar(axis, name)
        return ret
    
    @check_ctrl
    def set_axis_attr(self, axis, name, value):
        ctrl_info = self.ctrl_info
        axis_attr_info = ctrl_info.axis_attributes[name]
        try:
            return getattr(self.ctrl, axis_attr_info.fset)(axis, value)
        except AttributeError:
            return self.ctrl.SetAxisExtraPar(axis, name, value)
    
    @check_ctrl
    def set_ctrl_par(self, name, value):
        #return self.ctrl.setCtrlPar(unit, name, value)
        return self.ctrl.SetCtrlPar(name, value)
    
    @check_ctrl
    def get_ctrl_par(self, name):
        #return self.ctrl.getCtrlPar(unit, name, value)
        return self.ctrl.GetCtrlPar(name)
    
    @check_ctrl
    def set_axis_par(self, axis, name, value):
        #return self.ctrl.SetAxisPar(unit, axis, name, value)
        return self.ctrl.SetAxisPar(axis, name, value)
    
    @check_ctrl
    def get_axis_par(self, axis, name):
        #return self.ctrl.GetAxisPar(unit, axis, name, value)
        return self.ctrl.GetAxisPar(axis, name)
               

    # END API WHICH ACCESSES CONTROLLER API ------------------------------------
    
    # START API WHICH ACCESSES CRITICAL CONTROLLER API (like StateOne) ---------

    def __build_exc_info(self, ctrl_states, axes, exc_info):
        status = "".join(traceback.format_exception(*exc_info))
        state_info = State.Fault, status
        for axis in axes:
            element = self.get_element(axis=axis)
            ctrl_states[element] = state_info
    
    @staticmethod
    def _format_exception(exc_info):
        fmt_exc = traceback.format_exception_only(*exc_info[:2])
        fmt_exc = "".join(fmt_exc)
        if fmt_exc.endswith("\n"):
            fmt_exc = fmt_exc[:-1]
        return fmt_exc
    
    def raw_read_axis_states(self, axes=None, ctrl_states=None):
        """**Unsafe method**. Reads the state for the given axes. If axes
        is None, reads the state of all active axes.
        
        :param axes: the list of axis to get the state. Default is None meaning
                       all active axis in this controller
        :type axes: seq<int> or None
        :return:
            a tuple of two elements: a map containing the controller state
            information for each axis and a boolean telling if an error occured
        :rtype: dict<PoolElement, state info>, bool"""
        if axes is None:
            axes = self._element_axis.keys()
        if ctrl_states is None:
            ctrl_states = {}
        
        ctrl = self.ctrl
        
        try:
            ctrl.PreStateAll()
            for axis in axes:
                ctrl.PreStateOne(axis)
            ctrl.StateAll()
        except:
            exc_info = sys.exc_info()
            status = self._format_exception(exc_info)
            state_info = (State.Fault, status), exc_info
            for axis in axes:
                element = self.get_element(axis=axis)
                ctrl_states[element] = state_info
            return ctrl_states, True
        
        error = False
        for axis in axes:
            element = self.get_element(axis=axis)
            try:
                state_info = ctrl.StateOne(axis)
                if state_info is None:
                    raise Exception("%s.StateOne(%s(%d)) returns 'None'"
                                    % (self.name, element.name, axis))
                state_info = state_info, None
            except:
                exc_info = sys.exc_info()
                status = self._format_exception(exc_info)
                state_info = (State.Fault, status), exc_info
                error = True
            ctrl_states[element] = state_info
        return ctrl_states, error
    
    @check_ctrl
    def read_axis_states(self, axes=None):
        """Reads the state for the given axes. If axes is None, reads the
        state of all active axes.
        
        :param axes: the list of axis to get the state. Default is None
                       meaning all active axis in this controller
        :type axes: seq<int> or None
        :return: a map containing the controller state information for each axis
        :rtype: dict<PoolElement, state info>"""
        return self.raw_read_axis_states(axes=axes)
    
    def _read_axis_value(self, element):
        try:
            axis = element.get_axis()
            ctrl_value = self.ctrl.ReadOne(axis)
            if ctrl_value is None:
                msg = '%s.ReadOne(%s[%d]) return error: Expected value, ' \
                      'got None instead' % (self.name, element.name, axis)
                raise ValueError(msg)
            value = translate_ctrl_value(ctrl_value)
        except:
            value = SardanaValue(exc_info=sys.exc_info())
        return value
    
    def raw_read_axis_values(self, axes=None, ctrl_values=None):
        """**Unsafe method**. Reads the value for the given axes. If axes
        is None, reads the value of all active axes.
        
        :param axes: the list of axis to get the value. Default is None
                       meaning all active axis in this controller
        :type axes: seq<int> or None
        :return: a map containing the controller value information for each axis
        :rtype: dict<PoolElement, SardanaValue>"""
        if axes is None:
            axes = self._element_axis.keys()
        if ctrl_values is None:
            ctrl_values = {}
        
        ctrl = self.ctrl
        
        try:
            ctrl.PreReadAll()
            for axis in axes:
                ctrl.PreReadOne(axis)
            ctrl.ReadAll()
        except:
            exc_info = sys.exc_info()
            for axis in axes:
                element = self.get_element(axis=axis)
                ctrl_values[element] = None, exc_info
            return ctrl_values
            
        for axis in axes:
            element = self.get_element(axis=axis)
            ctrl_values[element] = self._read_axis_value(element)
        
        return ctrl_values

    @check_ctrl
    def read_axis_values(self, axes=None):
        """Reads the value for the given axes. If axes is None, reads the
        value of all active axes.
        
        :param axes: the list of axis to get the value. Default is None meaning
                       all active axis in this controller
        :type axes: seq<int> or None
        :return: a map containing the controller value information for each axis
        :rtype: dict<PoolElement, SardanaValue>"""
        return self.raw_read_axis_values(axes=axes)
    
    def raw_stop_all(self):
        try:
            return self._raw_stop_all()
        except:
            pass
    
    def raw_stop_one(self, axis):
        try:
            self._raw_stop_one(axis)
        except:
            pass
    
    def _raw_stop_all(self):
        try:
            return self.ctrl.StopAll()
        except:
            self.warning("StopAll() raises exception")
            self.debug("Details:", exc_info=1)
            raise
    
    def _raw_stop_one(self, axis):
        try:
            self.ctrl.StopOne(axis)
        except:
            self.warning("StopOne(%d) raises exception", axis)
            self.debug("Details:", exc_info=1)
            raise
    
    @check_ctrl
    def stop_all(self):
        self.raw_stop_all()
    
    stop = stop_all
    
    @check_ctrl
    def stop_one(self, axis):
        return self.raw_stop_one(axis)
    
    @check_ctrl
    def stop_axes(self, axes=None):
        """Stops the given axes. If axes is None, stops all active axes.
        
        :param axes: the list of axis to stop. Default is None
                       meaning all active axis in this controller
        :type axes: seq<int> or None
        """
        if axes is None:
            return self.raw_stop_all()
        
        for axis in axes:
            self.raw_stop_one(axis)
    
    @check_ctrl
    def stop_elements(self, elements=None):
        """Stops the given elements. If axes is None, stops all active axes.
        
        :param elements: the list of elements to stop. Default is None
                         meaning all active axis in this controller
        :type axes: seq<PoolElement> or None
        """
        if elements is None:
            return self.raw_stop_all()
        
        for element in elements:
            self.raw_stop_one(element.axis)
    
    def raw_abort_all(self):
        try:
            return self._raw_abort_all()
        except:
            pass
    
    def raw_abort_one(self, axis):
        try:
            self._raw_abort_one(axis)
        except:
            pass
    
    def _raw_abort_all(self):
        try:
            return self.ctrl.AbortAll()
        except:
            self.warning("AbortAll() raises exception")
            self.debug("Details:", exc_info=1)
            raise
    
    def _raw_abort_one(self, axis):
        try:
            self.ctrl.AbortOne(axis)
        except:
            self.warning("AbortOne(%d) raises exception", axis)
            self.debug("Details:", exc_info=1)
            raise

    @check_ctrl
    def abort_all(self):
        self.raw_abort_all()

    @check_ctrl
    def abort_one(self, axis):
        return self.raw_abort_one(axis)
    
    @check_ctrl
    def abort_axes(self, axes=None):
        """Aborts the given axes. If axes is None, aborts all active axes.
        
        :param axes: the list of axis to abort. Default is None
                       meaning all active axis in this controller
        :type axes: seq<int> or None
        """
        if axes is None:
            return self.raw_abort_all()
        
        for axis in axes:
            self.raw_abort_one(axis)
    
    @check_ctrl
    def abort_elements(self, elements=None):
        """Aborts the given elements. If axes is None, aborts all active axes.
        
        :param elements: the list of elements to abort. Default is None
                         meaning all active axis in this controller
        :type axes: seq<PoolElement> or None
        """
        if elements is None:
            return self.raw_abort_all()
        
        for element in elements:
            self.raw_abort_one(element.axis)
    
    abort = abort_all
    
    @check_ctrl
    def emergency_break(self, elements=None):
        """Stops the given elements. If axes is None, stops all active axes.
        If stop raises exception, an abort is attempted.
        
        :param elements: the list of elements to stop. Default is None
                         meaning all active axis in this controller
        :type axes: seq<PoolElement> or None
        """
        if elements is None:
            try:
                return self._raw_stop_all()
            except:
                return self.raw_abort_all()
        
        for element in elements:
            try:
                self._raw_stop_one(element.axis)
            except:
                self.raw_abort_one(element.axis)
    
    @check_ctrl
    def send_to_controller(self, stream):
        return self.ctrl.SendToCtrl(stream)
    
    # END API WHICH ACCESSES CRITICAL CONTROLLER API (like StateOne) -----------
    
    # START SPECIFIC TO MOTOR CONTROLLER ---------------------------------------
    
    def raw_move(self, axis_pos):
        ctrl = self.ctrl
        ctrl.PreStartAll()
        for axis, dial_position in axis_pos.items():
            ret = ctrl.PreStartOne(axis, dial_position)
            if not ret:
                raise Exception("%s.PreStartOne(%d, %f) returns False" \
                                % (self.name, axis, dial_position))
        
        for axis, dial_position in axis_pos.items():
            ctrl.StartOne(axis, dial_position)
        
        ctrl.StartAll()

    @check_ctrl
    def move(self, axis_pos):
        return self.raw_move(axis_pos)
    
    def has_backlash(self):
        return "Backlash" in self._ctrl.ctrl_features
    
    def wants_rounding(self):
        return "Rounding" in self._ctrl.ctrl_features

    @check_ctrl
    def define_position(self, axis, position):
        return self.ctrl.DefinePosition(axis, position)
        
    # END SPECIFIC TO MOTOR CONTROLLER -----------------------------------------

    # START SPECIFIC TO IOR CONTROLLER -----------------------------------------

    def write_one(self, axis, value):
        self.ctrl.WriteOne(axis, value)

    # END SPECIFIC TO IOR CONTROLLER -------------------------------------------


class PoolPseudoMotorController(PoolController):
    
    def __init__(self, **kwargs):
        self._motor_ids = kwargs.pop('role_ids')
        super(PoolPseudoMotorController, self).__init__(**kwargs)
    
    def serialize(self, *args, **kwargs):
        kwargs = PoolController.serialize(self, *args, **kwargs)
        kwargs['type'] = 'Controller'
        return kwargs
        
    def _create_ctrl_args(self):
        pars = PoolController._create_ctrl_args(self)
        kwargs = pars[4]
        kwargs['motor_ids'] = tuple(self._motor_ids)
        return pars

    def _read_axis_value(self, element):
        try:
            axis = element.get_axis()
            ctrl_value = self.ctrl.ReadOne(axis)
            if ctrl_value is None:
                msg = '%s.ReadOne(%s[%d]) return error: Expected value, ' \
                      'got None instead' % (self.name, element.name, axis)
                raise ValueError(msg)
            value = translate_ctrl_value(ctrl_value)
        except:
            value = SardanaValue(exc_info=sys.exc_info())
        return value
        
    @check_ctrl
    def calc_all_pseudo(self, physical_pos, curr_pseudo_pos):
        ctrl = self.ctrl
        try:
            ctrl_value = ctrl.CalcAllPseudo(physical_pos, curr_pseudo_pos)
            if ctrl_value is None:
                msg = '%s.CalcAllPseudo() return error: Expected value, ' \
                      'got None instead' % (self.name, )
                raise ValueError(msg)
            value = translate_ctrl_value(ctrl_value)
        except:
            value = SardanaValue(exc_info=sys.exc_info())
        return value

    @check_ctrl
    def calc_all_physical(self, pseudo_pos, curr_physical_pos):
        ctrl = self.ctrl
        try:
            ctrl_value = ctrl.CalcAllPhysical(pseudo_pos, curr_physical_pos)
            if ctrl_value is None:
                msg = '%s.CalcAllPhysical() return error: Expected value, ' \
                      'got None instead' % (self.name, )
                raise ValueError(msg)
            value = translate_ctrl_value(ctrl_value)
        except:
            value = SardanaValue(exc_info=sys.exc_info())
        return value

    @check_ctrl
    def calc_pseudo(self, axis, physical_pos, curr_pseudo_pos):
        ctrl = self.ctrl
        try:
            ctrl_value = ctrl.CalcPseudo(axis, physical_pos, curr_pseudo_pos)
            if ctrl_value is None:
                msg = '%s.CalcPseudo() return error: Expected value, ' \
                      'got None instead' % (self.name, )
                raise ValueError(msg)
            value = translate_ctrl_value(ctrl_value)
        except:
            value = SardanaValue(exc_info=sys.exc_info())
        return value

    @check_ctrl
    def calc_physical(self, axis, pseudo_pos, curr_physical_pos):
        ctrl = self.ctrl
        try:
            ctrl_value = ctrl.CalcPhysical(axis, pseudo_pos, curr_physical_pos)
            if ctrl_value is None:
                msg = '%s.CalcPhysical() return error: Expected value, ' \
                      'got None instead' % (self.name, )
                raise ValueError(msg)
            value = translate_ctrl_value(ctrl_value)
        except:
            value = SardanaValue(exc_info=sys.exc_info())
        return value


class PoolPseudoCounterController(PoolController):
    
    def __init__(self, **kwargs):
        self._counter_ids = kwargs.pop('role_ids')
        super(PoolPseudoCounterController, self).__init__(**kwargs)
    
    def serialize(self, *args, **kwargs):
        kwargs = PoolController.serialize(self, *args, **kwargs)
        kwargs['type'] = 'Controller'
        return kwargs
        
    def _create_ctrl_args(self):
        pars = PoolController._create_ctrl_args(self)
        kwargs = pars[4]
        kwargs['counter_ids'] = tuple(self._counter_ids)
        return pars
    
    @check_ctrl
    def calc(self, axis, values):
        ctrl = self.ctrl
        try:
            ctrl_value = ctrl.Calc(axis, values)
            if ctrl_value is None:
                msg = '%s.Calc() return error: Expected value, ' \
                      'got None instead' % (self.name, )
                raise ValueError(msg)
            value = translate_ctrl_value(ctrl_value)
        except:
            value = SardanaValue(exc_info=sys.exc_info())
        return value
    
    def calc_all(self, values):
        ctrl = self.ctrl
        try:
            ctrl_value = ctrl.CalcAll(values)
            if ctrl_value is None:
                msg = '%s.CalcAll() return error: Expected value, ' \
                      'got None instead' % (self.name, )
                raise ValueError(msg)
            value = translate_ctrl_value(ctrl_value)
        except:
            value = SardanaValue(exc_info=sys.exc_info())
        return value



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module is part of the Python Sardana libray. It defines the base classes
for sardana value translation"""

__all__ = ["CannotTranslateException",
           "register_controller_value_translator",
           "register_controller_state_translator",
           "ControllerValueTranslator", "ControllerStateTranslator"]

__docformat__ = 'restructuredtext'

from sardana.sardanavalue import SardanaValue 

__CTRL_STATE_TRANSLATORS = {}
__CTRL_VALUE_TRANSLATORS = {}


class CannotTranslateException(Exception):

    def __init__(self, *args, **kwargs):
        super(CannotTranslateException, self).__init__(*args, **kwargs)

    
class BaseControllerTranslator(object):
    
    def translate(self, value):
        raise CannotTranslateException
    
    def __call__(self, value):
        return self.translate(value)


class ControllerValueTranslator(BaseControllerTranslator):
    pass


class ControllerStateTranslator(BaseControllerTranslator):
    pass


def register_controller_value_translator(klass, *args, **kwargs):
    if not issubclass(klass, ControllerValueTranslator):
        raise Exception("Cannot register controller value translator. " \
                        "Class must inherit from ControllerValueTranslator")
    __CTRL_VALUE_TRANSLATORS[klass] = klass(*args, **kwargs)

def register_controller_state_translator(klass, *args, **kwargs):
    if not issubclass(klass, ControllerStateTranslator):
        raise Exception("Cannot register controller value translator. " \
                        "Class must inherit from ControllerStateTranslator")
    __CTRL_STATE_TRANSLATORS[klass] = klass(*args, **kwargs)

def translate_ctrl_value(value):
    if isinstance(value, SardanaValue):
        return value

    for _, translator in __CTRL_VALUE_TRANSLATORS.items():
        try:
            return translator(value)
        except CannotTranslateException:
            continue
    
    # Fallback translator
    return SardanaValue(value=value)
    
        



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""Pool utils"""

__all__ = ["PoolUtil"]

__docformat__ = 'restructuredtext'

from taurus.core.util import CaselessDict

class _PoolUtil(object):
    
    def __init__(self):
        self._ctrl_proxies = CaselessDict()
    
    def __call__(self, *args, **kwargs):
        return self
    
    def get_device(self, *args, **kwargs):
        ctrl_name = args[0]
        device_name = args[1]
        ctrl_devs = self._ctrl_proxies.get(ctrl_name)
        if ctrl_devs is None:
            self._ctrl_proxies[ctrl_name] = ctrl_devs = CaselessDict()
        dev = ctrl_devs.get(device_name)
        if dev is None:
            import PyTango
            ctrl_devs[device_name] = dev = PyTango.DeviceProxy(device_name)
        return dev
    
    get_motor = get_phy_motor = get_pseudo_motor = get_motor_group = \
        get_exp_channel = get_ct_channel = get_zerod_channel = get_oned_channel = \
        get_twod_channel = get_pseudo_counter_channel = get_measurement_group = \
        get_com_channel = get_ioregister = get_device

PoolUtil = _PoolUtil()


#!/usr/bin/env python

""" The sardana transformation tool.
    Syntax:
        python xls_to_sar.py <file.xml>
        
    This tool outputs a sardana XML file from an Excel XML file.
    
    file.xml must be a valid XML file comming from an EXCEL spreadsheet.
"""


import sys, os, types
from lxml import etree

def transform(f):
    directory = os.path.dirname(os.path.abspath(__file__))
    xslt_filename = os.path.join(directory, "XLS_TO_SAR.xslt")

    t = etree.XSLT(etree.parse(xslt_filename))
    if type(f) in types.StringTypes:
        doc = etree.parse(f)
    else:
        doc = f
    return t(doc)

def main():
    if len(sys.argv) < 2:
        print __doc__
        sys.exit(1)
        
    filename = sys.argv[1]
    t = transform(filename)
    print etree.tostring(t, pretty_print=True)

if __name__ == "__main__":
    main()



##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

import time
from sardana import State
from sardana.pool.controller import CounterTimerController


class Channel:
    
    def __init__(self,idx):
        self.idx = idx            # 1 based index
        self.value = 0.0
        self.is_counting = False
        self.active = True


class DummyCounterTimerController(CounterTimerController):
    "This class is the Tango Sardana CounterTimer controller for tests"

    gender = "Simulation"
    model  = "Basic"
    organization = "Sardana team"

    MaxDevice = 1024
    
    StoppedMode = 0
    TimerMode = 1
    MonitorMode = 2
    CounterMode = 3

    def __init__(self, inst, props, *args, **kwargs):
        CounterTimerController.__init__(self, inst, props, *args, **kwargs)
        self.channels = self.MaxDevice*[None,]
        self.reset()
        
    def reset(self):
        self.start_time = None
        self.integ_time = None
        self.monitor_count = None
        self.read_channels = {}
        self.counting_channels = {}
        
    def AddDevice(self,ind):
        idx = ind - 1
        self.channels[idx] = Channel(ind)
        
    def DeleteDevice(self,ind):
        idx = ind - 1
        self.channels[idx] = None

    def PreStateAll(self):
        pass
    
    def PreStateOne(self, ind):
        pass
    
    def StateAll(self):
        pass
    
    def StateOne(self, ind):
        idx = ind - 1
        sta = State.On
        status = "Stopped"
        if ind in self.counting_channels:
            channel = self.channels[idx]
            now = time.time()
            elapsed_time = now - self.start_time
            self._updateChannelState(ind, elapsed_time)
            if channel.is_counting:
                sta = State.Moving
                status = "Acquiring"
        return sta, status
        
    def _updateChannelState(self, ind, elapsed_time):
        if self.integ_time is not None:
            # counting in time
            if elapsed_time >= self.integ_time:
                self._finish(elapsed_time)
        elif self.monitor_count is not None:
            # monitor counts
            v = int(elapsed_time*100*ind)
            if v >= self.monitor_count:
                self._finish(elapsed_time)
    
    def _updateChannelValue(self, ind, elapsed_time):
        channel = self.channels[ind-1]
        if self.integ_time is not None:
            t = elapsed_time
            if not channel.is_counting:
                t = self.integ_time
            if ind == self._timer:
                channel.value = t
            else:
                channel.value = t * channel.idx
        elif self.monitor_count is not None:
            channel.value = int(elapsed_time*100*ind)
            if ind == self._monitor:
                if not channel.is_counting:
                    channel.value = self.monitor_count
    
    def _finish(self, elapsed_time, ind=None):
        if ind is None:
            for ind, channel in self.counting_channels.items():
                channel.is_counting = False
                self._updateChannelValue(ind, elapsed_time)
        else:
            if ind in self.counting_channels:
                channel = self.counting_channels[ind]
                channel.is_counting = False
                self._updateChannelValue(ind, elapsed_time)
            else:
                channel = self.channels[ind-1]
                channel.is_counting = False
        self.counting_channels = {}
                
    def PreReadAll(self):
        self.read_channels = {}
    
    def PreReadOne(self,ind):
        channel = self.channels[ind-1]
        self.read_channels[ind] = channel

    def ReadAll(self):
        # if in acquisition then calculate the values to return
        if self.counting_channels:
            now = time.time()
            elapsed_time = now - self.start_time
            for ind, channel in self.read_channels.items():
                self._updateChannelState(ind, elapsed_time)
                if channel.is_counting:
                    self._updateChannelValue(ind, elapsed_time)
    
    def ReadOne(self, ind):
        v = self.read_channels[ind].value
        return v
    
    def PreStartAll(self):
        self.counting_channels = {}
    
    def PreStartOne(self, ind, value=None):
        idx = ind - 1
        channel = self.channels[idx]
        channel.value = 0.0
        self.counting_channels[ind] = channel
        return True
    
    def StartOne(self, ind, value=None):
        self.counting_channels[ind].is_counting = True
    
    def StartAll(self):
        self.start_time = time.time()
    
    def LoadOne(self, ind, value):
        if value > 0:
            self.integ_time = value
            self.monitor_count = None
        else:
            self.integ_time = None
            self.monitor_count = -value
    
    def AbortOne(self, ind):
        now = time.time()
        if ind in self.counting_channels:
            elapsed_time = now - self.start_time
            self._finish(elapsed_time, ind=ind)
            



#!/usr/bin/env python
# -*- coding: utf-8 -*-

##############################################################################
##
## This file is part of Sardana
## 
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This package provides the spock generic utilities"""

def translate_version_str2list(version_str):
    """Translates a version string in format 'x[.y[.z[...]]]' into a list of
    numbers"""
    if version_str is None:
        ver = [0, 0]
    else:
        ver = []
        for i in version_str.split(".")[:2]:
            try:
                i = int(i)
            except:
                i = 0
            ver.append(i)
    return ver

def get_ipython_version():
    """Returns the current IPython version"""
    import IPython
    v = None
    try:
        try:
            v = IPython.Release.version
        except:
            try:
                v = IPython.release.version
            except:
                pass
    except:
        pass
    return v

def get_ipython_version_list():
    ipv_str = get_ipython_version()
    return translate_version_str2list(ipv_str)

ipv = get_ipython_version_list()
if ipv >= [0, 10] and ipv < [0, 11]:
    from ipython_00_10.genutils import *
elif ipv >= [0, 11] and ipv < [1, 0]:
    from ipython_00_11.genutils import *
else:
    from ipython_01_00.genutils import *





#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module contains the main pool class"""

from __future__ import print_function
from __future__ import with_statement

__all__ = ["Pool"]

__docformat__ = 'restructuredtext'

import os.path
import logging.handlers

from taurus.core.taurusvalidator import AttributeNameValidator
from taurus.core.util.containers import CaselessDict

from sardana import InvalidId, ElementType, TYPE_ACQUIRABLE_ELEMENTS, \
    TYPE_PSEUDO_ELEMENTS, TYPE_PHYSICAL_ELEMENTS, TYPE_MOVEABLE_ELEMENTS
from sardana.sardanamanager import SardanaElementManager, SardanaIDManager
from sardana.sardanamodulemanager import ModuleManager
from sardana.sardanaevent import EventType

from .poolobject import PoolObject
from .poolcontainer import PoolContainer
from .poolcontroller import PoolController
from .poolmonitor import PoolMonitor
from .poolmetacontroller import TYPE_MAP_OBJ
from .poolcontrollermanager import ControllerManager


class Graph(dict):

    def find_path(self, start, end, path=[]):
        path = path + [start]
        if start == end:
            return path
        if start not in self:
            return None
        for node in self[start]:
            if node not in path:
                newpath = self.find_path(node, end, path)
                if newpath: return newpath
        return None

    def find_all_paths(self, start, end, path=[]):
        path = path + [start]
        if start == end:
            return [path]
        if start not in self:
            return []
        paths = []
        for node in self[start]:
            if node not in path:
                newpaths = self.find_all_paths(node, end, path)
                for newpath in newpaths:
                    paths.append(newpath)
        return paths

    def find_shortest_path(self, start, end, path=[]):
        path = path + [start]
        if start == end:
            return path
        if start not in self:
            return None
        shortest = None
        for node in self[start]:
            if node not in path:
                newpath = self.find_shortest_path(node, end, path)
                if newpath:
                    if not shortest or len(newpath) < len(shortest):
                        shortest = newpath
        return shortest

        
class Pool(PoolContainer, PoolObject, SardanaElementManager, SardanaIDManager):
    """The central pool class."""

    #: Default value representing the number of state reads per position
    #: read during a motion loop
    Default_MotionLoop_StatesPerPosition = 10

    #: Default value representing the sleep time for each motion loop
    Default_MotionLoop_SleepTime = 0.01

    #: Default value representing the number of state reads per value
    #: read during a motion loop
    Default_AcqLoop_StatesPerValue = 10

    #: Default value representing the sleep time for each acquisition loop
    Default_AcqLoop_SleepTime = 0.01

    Default_DriftCorrection = True

    def __init__(self, full_name, name=None):
        self._path_id = None
        self._motion_loop_states_per_position = self.Default_MotionLoop_StatesPerPosition
        self._motion_loop_sleep_time = self.Default_MotionLoop_SleepTime
        self._acq_loop_states_per_value = self.Default_AcqLoop_StatesPerValue
        self._acq_loop_sleep_time = self.Default_AcqLoop_SleepTime
        self._drift_correction = self.Default_DriftCorrection
        self._remote_log_handler = None
        
        # dict<str, dict<str, str>>
        # keys are acquisition channel names and value is a dict describing the
        # channel containing:
        #  - 'name': with value being the channel name (given by user)
        #  - 'full_name': acq channel full name (ex: tango attribute)
        #  - 'origin': 'local' if local to this server or 'remote' if a remote
        #    channel
        self._extra_acquisition_element_names = CaselessDict()

        PoolContainer.__init__(self)
        PoolObject.__init__(self, full_name=full_name, name=name, id=InvalidId,
                            pool=self, elem_type=ElementType.Pool)
        self._monitor = PoolMonitor(self, "PMonitor", auto_start=False)
        #self.init_local_logging()
        ControllerManager().set_pool(self)
    
    # TODO: not ready to use. path must be the same as the one calculated in
    # sardana.tango.core.util:prepare_logging 
    def init_local_logging(self):
        log = logging.getLogger("Controller")
        log.propagate = 0
        path = os.path.join(os.sep, "tmp", "tango")
        log_file_name = os.path.join(path, 'controller.log.txt')
        try:
            if not os.path.exists(path):
                os.makedirs(path, 0777)
            f_h = logging.handlers.RotatingFileHandler(log_file_name,
                                                       maxBytes=1E7,
                                                       backupCount=5)

            f_h.setFormatter(self.getLogFormat())
            log.addHandler(f_h)
            self.info("Controller logs stored in %s", log_file_name)
        except:
            self.warning("Controller logs could not be created!")
            self.debug("Details:", exc_info=1)
        
    def clear_remote_logging(self):
        rh = self._remote_log_handler
        if rh is None:
            return
        log = logging.getLogger("Controller")
        log.removeHandler(rh)
        self._remote_log_handler = None
        
    def init_remote_logging(self, host=None, port=None):
        """Initializes remote logging.

        :param host: host name [default: None, meaning use the machine host name
                     as returned by :func:`socket.gethostname`].
        :type host: str
        :param port: port number [default: None, meaning use
                     :data:`logging.handlers.DEFAULT_TCP_LOGGING_PORT`"""
        log = logging.getLogger("Controller")

        # port 0 means no remote logging
        if port == 0:
            return

        # first check that the handler has not been initialized yet
        for handler in log.handlers:
            if isinstance(handler, logging.handlers.SocketHandler):
                return
        if host is None:
            import socket
            host = socket.gethostname()
            #host = socket.getfqdn()
        if port is None:
            port = logging.handlers.DEFAULT_TCP_LOGGING_PORT
        handler = logging.handlers.SocketHandler(host, port)
        if hasattr(handler, 'retryMax'):
            # default max retry is 30s which seems too much. Let's make it that
            # the pool tries to reconnect to a client every 10s (similar to the
            # tango event reconnection
            handler.retryMax = 10.0
        log.addHandler(handler)
        self.info("Remote logging initialized for host '%s' on port %d",
                  host, port)

    def serialize(self, *args, **kwargs):
        kwargs = PoolObject.serialize(self, *args, **kwargs)
        kwargs['type'] = self.__class__.__name__
        kwargs['id'] = InvalidId
        kwargs['parent'] = None
        return kwargs

    def set_motion_loop_sleep_time(self, motion_loop_sleep_time):
        self._motion_loop_sleep_time = motion_loop_sleep_time

    def get_motion_loop_sleep_time(self):
        return self._motion_loop_sleep_time

    motion_loop_sleep_time = property(get_motion_loop_sleep_time,
                                      set_motion_loop_sleep_time,
                                      doc="motion sleep time (s)")

    def set_motion_loop_states_per_position(self, motion_loop_states_per_position):
        self._motion_loop_states_per_position = motion_loop_states_per_position

    def get_motion_loop_states_per_position(self):
        return self._motion_loop_states_per_position

    motion_loop_states_per_position = property(get_motion_loop_states_per_position,
        set_motion_loop_states_per_position,
        doc="Number of State reads done before doing a position read in the "
            "motion loop")

    def set_acq_loop_sleep_time(self, acq_loop_sleep_time):
        self._acq_loop_sleep_time = acq_loop_sleep_time

    def get_acq_loop_sleep_time(self):
        return self._acq_loop_sleep_time

    acq_loop_sleep_time = property(get_acq_loop_sleep_time,
                                   set_acq_loop_sleep_time,
                                   doc="acquisition sleep time (s)")

    def set_acq_loop_states_per_value(self, acq_loop_states_per_value):
        self._acq_loop_states_per_value = acq_loop_states_per_value

    def get_acq_loop_states_per_value(self):
        return self._acq_loop_states_per_value

    acq_loop_states_per_value = property(get_acq_loop_states_per_value,
        set_acq_loop_states_per_value,
        doc="Number of State reads done before doing a value read in the "
            "acquisition loop")

    def set_drift_correction(self, drift_correction):
        self._drift_correction = drift_correction

    def get_drift_correction(self):
        return self._drift_correction

    drift_correction = property(get_drift_correction,
                                set_drift_correction,
                                doc="drift correction")
    @property
    def monitor(self):
        return self._monitor

    @property
    def ctrl_manager(self):
        return ControllerManager()

    def set_python_path(self, path):
        mod_man = ModuleManager()
        if self._path_id is not None:
            mod_man.remove_python_path(self._path_id)
        self._path_id = mod_man.add_python_path(path)

    def set_path(self, path):
        self.ctrl_manager.setControllerPath(path, reload=False)

    def get_controller_libs(self):
        return self.ctrl_manager.getControllerLibs()

    def get_controller_lib_names(self):
        return self.ctrl_manager.getControllerLibNames()

    def get_controller_class_names(self):
        return self.ctrl_manager.getControllerNames()

    def get_controller_classes(self):
        return self.ctrl_manager.getControllers()

    def get_controller_class_info(self, name):
        return self.ctrl_manager.getControllerMetaClass(name)

    def get_controller_classes_info(self, names):
        return self.ctrl_manager.getControllerMetaClasses(names)

    def get_controller_libs_summary_info(self):
        libs = self.get_controller_libs()
        ret = []
        for ctrl_lib_info in libs:
            elem = "%s (%s)" % (ctrl_lib_info.getName(), ctrl_lib_info.getFileName())
            ret.append(elem)
        return ret

    def get_controller_classes_summary_info(self):
        ctrl_classes = self.get_controller_classes()
        ret = []
        for ctrl_class_info in ctrl_classes:
            types = ctrl_class_info.getTypes()
            types_str = [ TYPE_MAP_OBJ[t].name for t in types if t != ElementType.Controller ]
            types_str = ", ".join(types_str)
            elem = "%s (%s) %s" % (ctrl_class_info.getName(), ctrl_class_info.getFileName(), types_str)
            ret.append(elem)
        return ret

    def get_elements_str_info(self, obj_type=None):
        if obj_type is None:
            objs = self.get_element_id_map().values()
            objs.extend(self.get_controller_classes())
            objs.extend(self.get_controller_libs())
        elif obj_type == ElementType.ControllerClass:
            objs = self.get_controller_classes()
        elif obj_type == ElementType.ControllerLibrary:
            objs = self.get_controller_libs()
        else:
            objs = self.get_elements_by_type(obj_type)
        name = self.full_name
        return [ obj.str(pool=name) for obj in objs ]

    def get_elements_info(self, obj_type=None):
        if obj_type is None:
            objs = self.get_element_id_map().values()
            objs.extend(self.get_controller_classes())
            objs.extend(self.get_controller_libs())
            objs.append(self)
        elif obj_type == ElementType.ControllerClass:
            objs = self.get_controller_classes()
        elif obj_type == ElementType.ControllerLibrary:
            objs = self.get_controller_libs()
        else:
            objs = self.get_elements_by_type(obj_type)
        name = self.full_name
        return [ obj.serialize(pool=name) for obj in objs ]

    def get_acquisition_elements_info(self):
        ret = []
        for _, element in self.get_element_name_map().items():
            if element.get_type() not in TYPE_ACQUIRABLE_ELEMENTS:
                continue
            acq_channel = element.get_default_acquisition_channel()
            full_name = "{0}/{1}".format(element.full_name, acq_channel)
            info = dict(name=element.name, full_name=full_name, origin='local')
            ret.append(info)
        ret.extend(self._extra_acquisition_element_names.values())
        return ret

    def get_acquisition_elements_str_info(self):
        return map(self.str_object, self.get_acquisition_elements_info())

    def create_controller(self, **kwargs):
        ctrl_type = kwargs['type']
        lib = kwargs['library']
        class_name = kwargs['klass']
        name = kwargs['name']
        elem_type = ElementType[ctrl_type]
        mod_name, _ = os.path.splitext(lib)
        kwargs['module'] = mod_name

        td = TYPE_MAP_OBJ[ElementType.Controller]
        klass_map = td.klass
        auto_full_name = td.auto_full_name
        kwargs['full_name'] = full_name = \
            kwargs.get("full_name", auto_full_name.format(**kwargs))
        self.check_element(name, full_name)

        ctrl_class_info = None
        ctrl_lib_info = self.ctrl_manager.getControllerLib(mod_name)
        if ctrl_lib_info is not None:
            ctrl_class_info = ctrl_lib_info.get_controller(class_name)

        kwargs['pool'] = self
        kwargs['class_info'] = ctrl_class_info
        kwargs['lib_info'] = ctrl_lib_info
        eid = kwargs.get('id')
        if eid is None:
            kwargs['id'] = eid = self.get_new_id()
        else:
            self.reserve_id(eid)

        # For pseudo controllers make sure 'role_ids' is given
        klass = klass_map.get(elem_type, PoolController)
        if elem_type in TYPE_PSEUDO_ELEMENTS:
            motor_roles = kwargs['role_ids']

        # make sure the properties (that may have come from a case insensitive
        # environment like tango) are made case sensitive
        props = {}
        if ctrl_class_info is None:
            ctrl_prop_info = {}
        else:
            ctrl_prop_info = ctrl_class_info.ctrl_properties
        for k, v in kwargs['properties'].items():
            info = ctrl_prop_info.get(k)
            if info is None:
                props[k] = v
            else:
                props[info.name] = v
        kwargs['properties'] = props

        ctrl = klass(**kwargs)
        ret = self.add_element(ctrl)
        self.fire_event(EventType("ElementCreated"), ctrl)
        return ret

    def create_element(self, **kwargs):
        etype = kwargs['type']
        ctrl_id = kwargs['ctrl_id']
        axis = kwargs['axis']
        elem_type = ElementType[etype]
        name = kwargs['name']

        try:
            ctrl = self.get_element(id=ctrl_id)
        except:
            raise Exception("No controller with id '%d' found" % ctrl_id)

        elem_axis = ctrl.get_element(axis=axis)
        if elem_axis is not None:
            raise Exception("Controller already contains axis %d (%s)"
                            % (axis, elem_axis.get_name()))

        kwargs['pool'] = self
        kwargs['ctrl'] = ctrl
        kwargs['ctrl_name'] = ctrl.get_name()

        td = TYPE_MAP_OBJ[elem_type]
        klass = td.klass
        auto_full_name = td.auto_full_name
        full_name = kwargs.get("full_name", auto_full_name.format(**kwargs))

        self.check_element(name, full_name)

        if ctrl.is_online():
            ctrl_types, ctrl_id = ctrl.get_ctrl_types(), ctrl.get_id()
            if elem_type not in ctrl_types:
                ctrl_type_str = ElementType.whatis(ctrl_types[0])
                raise Exception("Cannot create %s in %s controller"
                                % (etype, ctrl_type_str))

        #check if controller is online
        #check if axis is allowed
        #create the element in the controller

        eid = kwargs.get('id')
        if eid is None:
            kwargs['id'] = eid = self.get_new_id()
        else:
            self.reserve_id(eid)
        elem = klass(**kwargs)
        ctrl.add_element(elem)
        ret = self.add_element(elem)
        self.fire_event(EventType("ElementCreated"), elem)
        return ret

    def create_motor_group(self, **kwargs):
        name = kwargs['name']
        elem_ids = kwargs["user_elements"]

        kwargs['pool'] = self
        kwargs["pool_name"] = self.name
        td = TYPE_MAP_OBJ[ElementType.MotorGroup]
        klass = td.klass
        auto_full_name = td.auto_full_name
        full_name = kwargs.get("full_name", auto_full_name.format(**kwargs))
        kwargs.pop('pool_name')

        self.check_element(name, full_name)

        for elem_id in elem_ids:
            elem = self.pool.get_element(id=elem_id)
            if elem.get_type() not in (ElementType.Motor, ElementType.PseudoMotor):
                raise Exception("%s is not a motor" % elem.name)

        eid = kwargs.get('id')
        if eid is None:
            kwargs['id'] = eid = self.get_new_id()
        else:
            self.reserve_id(eid)

        elem = klass(**kwargs)

        ret = self.add_element(elem)
        self.fire_event(EventType("ElementCreated"), elem)
        return ret

    def create_measurement_group(self, **kwargs):
        name = kwargs['name']
        elem_ids = kwargs["user_elements"]

        kwargs['pool'] = self
        kwargs["pool_name"] = self.name

        td = TYPE_MAP_OBJ[ElementType.MeasurementGroup]
        klass = td.klass
        auto_full_name = td.auto_full_name

        full_name = kwargs.get("full_name", auto_full_name.format(**kwargs))
        kwargs.pop('pool_name')

        self.check_element(name, full_name)

        for elem_id in elem_ids:
            if type(elem_id) is int:
                self.pool.get_element(id=elem_id)
            else:
                tg_attr_validator = AttributeNameValidator()
                params = tg_attr_validator.getParams(elem_id)
                if params is None:
                    raise Exception("Invalid channel name %s" % elem_id)

        eid = kwargs.get('id')
        if eid is None:
            kwargs['id'] = eid = self.get_new_id()
        else:
            self.reserve_id(eid)

        elem = klass(**kwargs)

        ret = self.add_element(elem)
        self.fire_event(EventType("ElementCreated"), elem)
        return ret

    def delete_element(self, name):
        try:
            elem = self.get_element(name=name)
        except:
            try:
                elem = self.get_element(full_name=name)
            except:
                raise Exception("There is no element with name '%s'" % name)

        elem_type = elem.get_type()
        if elem_type == ElementType.Controller:
            if len(elem.get_elements()) > 0:
                raise Exception("Cannot delete controller with elements. "
                                "Delete elements first")
        elif elem_type == ElementType.Instrument:
            if elem.has_instruments():
                raise Exception("Cannot delete instrument with instruments. "
                                "Delete instruments first")
            if elem.has_elements():
                raise Exception("Cannot delete instrument with elements")
            parent_instrument = elem.parent_instrument
            if parent_instrument is not None:
                parent_instrument.remove_instrument(elem)
        elif hasattr(elem, "get_controller"):
            ctrl = elem.get_controller()
            ctrl.remove_element(elem)
            instrument = elem.instrument
            if instrument is not None:
                instrument.remove_element(elem)

        self.remove_element(elem)

        self.fire_event(EventType("ElementDeleted"), elem)

    def create_instrument(self, full_name, klass_name, id=None):
        is_root = full_name.count('/') == 1

        if is_root:
            parent_full_name, _ = '', full_name[1:]
            parent = None
        else:
            parent_full_name, _ = full_name.rsplit('/', 1)
            try:
                parent = self.get_element_by_full_name(parent_full_name)
            except:
                raise Exception("No parent instrument named '%s' found"
                                % parent_full_name)
            if parent.get_type() != ElementType.Instrument:
                raise Exception("%s is not an instrument as expected"
                                % parent_full_name)

        self.check_element(full_name, full_name)

        td = TYPE_MAP_OBJ[ElementType.Instrument]
        klass = td.klass

        if id is None:
            id = self.get_new_id()
        else:
            self.reserve_id(id)
        elem = klass(id=id, name=full_name, full_name=full_name,
                     parent=parent, klass=klass_name, pool=self)
        if parent:
            parent.add_instrument(elem)
        ret = self.add_element(elem)
        self.fire_event(EventType("ElementCreated"), elem)
        return ret

    def stop(self):
        controllers = self.get_elements_by_type(ElementType.Controller)
        for controller in controllers:
            controller.stop_all()

    def abort(self):
        controllers = self.get_elements_by_type(ElementType.Controller)
        for controller in controllers:
            controller.abort_all()

    # --------------------------------------------------------------------------
    # (Re)load code
    # --------------------------------------------------------------------------

    def reload_controller_lib(self, lib_name):
        manager = self.ctrl_manager

        old_lib = manager.getControllerLib(lib_name)
        new_elements, changed_elements, deleted_elements = [], [], []
        old_ctrl_classes = ()
        if old_lib is not None:
            ctrl_infos = old_lib.get_controllers()
            pool_ctrls = self.get_elements_by_type(ElementType.Controller)
            init_pool_ctrls = []
            for pool_ctrl in pool_ctrls:
                if pool_ctrl.get_ctrl_info() in ctrl_infos:
                    init_pool_ctrls.append(pool_ctrl)
            old_ctrl_classes = ctrl_infos
            changed_elements.append(old_lib)

        new_lib = manager.reloadControllerLib(lib_name)

        if old_lib is None:
            new_elements.extend(new_lib.get_controllers())
            new_elements.append(new_lib)
        else:
            new_names = set([ ctrl.name for ctrl in new_lib.get_controllers() ])
            old_names = set([ ctrl.name for ctrl in old_lib.get_controllers() ])
            changed_names = set.intersection(new_names, old_names)
            deleted_names = old_names.difference(new_names)
            new_names = new_names.difference(old_names)

            for new_name in new_names:
                new_elements.append(new_lib.get_controller(new_name))
            for changed_name in changed_names:
                changed_elements.append(new_lib.get_controller(changed_name))
            for deleted_name in deleted_names:
                deleted_elements.append(old_lib.get_controller(deleted_name))

        evt = { "new" : new_elements, "change" : changed_elements,
                "del" : deleted_elements }

        self.fire_event(EventType("ElementsChanged"), evt)

        if old_lib is not None:
            for pool_ctrl in init_pool_ctrls:
                pool_ctrl.re_init()

    def reload_controller_class(self, class_name):
        ctrl_info = self.ctrl_manager.getControllerMetaClass(class_name)
        lib_name = ctrl_info.module_name
        self.reload_controller_lib(lib_name)

    def get_element_id_graph(self):
        physical_elems_id_map = {}
        elem_type_map = self.get_element_type_map()
        for elem_type in TYPE_PHYSICAL_ELEMENTS:
            physical_elems_id_map.update(elem_type_map[elem_type])
        #TODO
        
    def _build_element_id_dependencies(self, elem_id, graph=None):
        if graph is None:
            graph = Graph()
        elem = self.get_element_by_id(elem_id)
        if elem.get_id() in graph or elem.get_type() in TYPE_PHYSICAL_ELEMENTS:
            return graph
        graph[elem_id] = list(elem.get_user_element_ids())
        return graph
        
    def get_moveable_id_graph(self):
        moveable_elems_id_map = {}
        elem_type_map = self.get_element_type_map()
        for elem_type in TYPE_MOVEABLE_ELEMENTS:
            moveable_elems_id_map.update(elem_type_map[elem_type])
        graph = Graph()
        for moveable_id in moveable_elems_id_map:
            self._build_element_id_dependencies(moveable_id, graph)
        return graph

    def _build_element_dependencies(self, elem, graph=None):
        if graph is None:
            graph = Graph()
        if elem.get_id() in graph or elem.get_type() in TYPE_PHYSICAL_ELEMENTS:
            return graph
        graph[elem] = list(elem.get_user_elements())
        return graph
        
    def get_moveable_graph(self):
        moveable_elems_map = {}
        elem_type_map = self.get_element_type_map()
        for elem_type in TYPE_MOVEABLE_ELEMENTS:
            moveable_elems_map.update(elem_type_map[elem_type])
        graph = Graph()
        for moveable in moveable_elems_map.values():
            self._build_element_dependencies(moveable, graph)
        return graph    



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

""" """

__docformat__ = 'restructuredtext'

__all__ = ["get_tango_version_str", "get_tango_version_number",
           "get_pytango_version_str", "get_pytango_version_number",
           "exception_str",
           "GenericScalarAttr", "GenericSpectrumAttr", "GenericImageAttr",
           "memorize_write_attribute",
           "tango_protect", "to_tango_state", "to_tango_type_format",
           "to_tango_type", "to_tango_access", "to_tango_attr_info",
           "from_tango_access", "from_tango_type_format",
           "from_tango_state_to_state",
           "from_deviceattribute_value", "from_deviceattribute",
           "throw_sardana_exception",
           "prepare_tango_logging", "prepare_rconsole", "run_tango_server",
           "run"]

import sys
import os.path
import string
import traceback
import time
import logging

import PyTango
from PyTango import Util, Database, WAttribute, DbDevInfo, DevFailed, \
    DevVoid, DevLong, DevBoolean, DevString, DevDouble, \
    DevState, SCALAR, SPECTRUM, IMAGE, FMT_UNKNOWN, \
    READ_WRITE, READ, Attr, SpectrumAttr, ImageAttr, \
    DeviceClass, Except

import taurus
from taurus.core.util.log import Logger

import sardana
from sardana import State, SardanaServer, DataType, DataFormat, InvalidId, \
    DataAccess, to_dtype_dformat, to_daccess, Release, ServerRunMode
from sardana.sardanaexception import SardanaException
from sardana.sardanavalue import SardanaValue
from sardana.util.wrap import wraps
from sardana.pool.poolmetacontroller import DataInfo


NO_DB_MAP = {
    "Pool" : (
        ("pool_demo", "sardana/pool/demo", dict(Version="1.0.0"), ),
    ),
    "Controller" : (
        ("motctrl", "controller/dummymotorcontroller/motctrl",
         dict(Id=1, Type="Motor", Klass="DummyMotorController",
              Library="DummyMotorController.py", Role_ids=(), ), ),
        ("iorctrl", "controller/dummyiorcontroller/iorctrl",
         dict(Id=2, Type="IORegister", Klass="DummyIORController",
              Library="DummyIORController.py", Role_ids=(), ), ),
        ("ctctrl", "controller/dummycountertimercontroller/ctctrl",
         dict(Id=3, Type="CTExpChannel", Klass="DummyCounterTimerController",
              Library="DummyCounterTimerController.py", Role_ids=(), ), ),
        ("zerodctrl", "controller/dummyzerodcontroller/zerodctrl",
         dict(Id=4, Type="ZeroDExpChannel", Klass="DummyZeroDController",
              Library="DummyZeroDController.py", Role_ids=(), ), ),
        ("slitctrl", "controller/slit/slitctrl",
         dict(Id=5, Type="PseudoMotor", Klass="Slit",
              Library="Slit.py", Role_ids=(), ), ),
        ("ioi0ctrl", "controller/ioveri0/ioi0ctrl",
         dict(Id=6, Type="PseudoCounter", Klass="IoverI0",
              Library="IoverI0.py", Role_ids=(), ), ),
    ),
    "Motor" : (
        ("slt",  "motor/motctrl/1",  dict(Id=101, Axis=1,  Ctrl_id=1, Instrument_id=InvalidId, Sleep_bef_last_read=0), ),
        ("slb",  "motor/motctrl/2",  dict(Id=102, Axis=2,  Ctrl_id=1, Instrument_id=InvalidId, Sleep_bef_last_read=0), ),
        ("mot1", "motor/motctrl/3",  dict(Id=103, Axis=3,  Ctrl_id=1, Instrument_id=InvalidId, Sleep_bef_last_read=0), ),
        ("mot2", "motor/motctrl/4",  dict(Id=104, Axis=4,  Ctrl_id=1, Instrument_id=InvalidId, Sleep_bef_last_read=0), ),
        ("mot3", "motor/motctrl/5",  dict(Id=105, Axis=5,  Ctrl_id=1, Instrument_id=InvalidId, Sleep_bef_last_read=0), ),
        ("mot4", "motor/motctrl/6",  dict(Id=106, Axis=6,  Ctrl_id=1, Instrument_id=InvalidId, Sleep_bef_last_read=0), ),
        ("th",   "motor/motctrl/7",  dict(Id=107, Axis=7,  Ctrl_id=1, Instrument_id=InvalidId, Sleep_bef_last_read=0), ),
        ("tth",  "motor/motctrl/8",  dict(Id=108, Axis=8,  Ctrl_id=1, Instrument_id=InvalidId, Sleep_bef_last_read=0), ),
        ("chi",  "motor/motctrl/9",  dict(Id=109, Axis=9,  Ctrl_id=1, Instrument_id=InvalidId, Sleep_bef_last_read=0), ),
        ("phi",  "motor/motctrl/10", dict(Id=110, Axis=10, Ctrl_id=1, Instrument_id=InvalidId, Sleep_bef_last_read=0), ),
    ),
    "IORegister" : (
        ("ior1", "ioregister/iorctrl/1", dict(Id=201, Axis=1, Ctrl_id=2, Instrument_id=InvalidId), ),
        ("ior2", "ioregister/iorctrl/2", dict(Id=202, Axis=2, Ctrl_id=2, Instrument_id=InvalidId), ),
        ("ior3", "ioregister/iorctrl/3", dict(Id=203, Axis=3, Ctrl_id=2, Instrument_id=InvalidId), ),
        ("ior4", "ioregister/iorctrl/4", dict(Id=204, Axis=4, Ctrl_id=2, Instrument_id=InvalidId), ),
    ),
    "CTExpChannel" : (
        ("ct1", "expchan/ctctrl/1", dict(Id=301, Axis=1, Ctrl_id=3, Instrument_id=InvalidId), ),
        ("ct2", "expchan/ctctrl/2", dict(Id=302, Axis=2, Ctrl_id=3, Instrument_id=InvalidId), ),
        ("ct3", "expchan/ctctrl/3", dict(Id=303, Axis=3, Ctrl_id=3, Instrument_id=InvalidId), ),
        ("ct4", "expchan/ctctrl/4", dict(Id=304, Axis=4, Ctrl_id=3, Instrument_id=InvalidId), ),
    ),
    "ZeroDExpChannel" : (
        ("zerod1", "expchan/zerodctrl/1", dict(Id=401, Axis=1, Ctrl_id=4, Instrument_id=InvalidId), ),
        ("zerod2", "expchan/zerodctrl/2", dict(Id=402, Axis=2, Ctrl_id=4, Instrument_id=InvalidId), ),
        ("zerod3", "expchan/zerodctrl/3", dict(Id=403, Axis=3, Ctrl_id=4, Instrument_id=InvalidId), ),
        ("zerod4", "expchan/zerodctrl/4", dict(Id=404, Axis=4, Ctrl_id=4, Instrument_id=InvalidId), ),
    ),
    "PseudoMotor" : (
        ("gap",    "pm/slitctrl/1",  dict(Id=501, Axis=1,  Ctrl_id=5, Instrument_id=InvalidId, Elements=("101", "102",) ), ),
        ("offset", "pm/slitctrl/2",  dict(Id=502, Axis=2,  Ctrl_id=5, Instrument_id=InvalidId, Elements=("101", "102",) ), ),
    ),
    "PseudoCounter" : (
        ("inorm", "pc/ioi0ctrl/1",  dict(Id=601, Axis=1,  Ctrl_id=6, Instrument_id=InvalidId, Elements=("301", "302",) ), ),
    ),
    "MotorGroup" : (
        ("motgrp1", "mg/pool_demo/motgrp1",  dict(Id=701, Elements=("103", "104",) ), ),
    ),
    "MeasurementGroup" : (
        ("mntgrp1", "mntgrp/pool_demo/mntgrp1",  dict(Id=701, Elements=("301", "302", "303", "401",) ), ),
    ),
    "MacroServer" : (
        ("MS_demo", "sardana/ms/demo",  dict(PoolNames=["sardana/pool/demo"] ), ),
    ),
    "Door" : (
        ("Door_demo", "sardana/door/demo",
         dict(Id=1001, MacroServerName="sardana/ms/demo", MaxMsgBufferSize=512), ),
    ),
}

#-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
# PyTango utilities
#-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

def get_pytango_version_str():
    try:
        import PyTango
    except:
        return None
    try:
        return PyTango.Release.version
    except:
        return '0.0.0'

def get_pytango_version_number():
    tgver_str = get_pytango_version_str()
    if tgver_str is None:
        return None
    import sardana.sardanautils
    return sardana.sardanautils.translate_version_str2int(tgver_str)

def get_tango_version_str():
    try:
        import PyTango.constants
    except:
        return None
    try:
        return PyTango.constants.TgLibVers
    except:
        return '0.0.0'
    
def get_tango_version_number():
    tgver_str = get_tango_version_str()
    if tgver_str is None:
        return None
    import sardana.sardanautils
    return sardana.sardanautils.translate_version_str2int(tgver_str)

class GenericScalarAttr(Attr):
    pass


class GenericSpectrumAttr(SpectrumAttr):
    
    def __init__(self, name, tg_type, tg_access, dim_x=2048):
        SpectrumAttr.__init__(self, name, tg_type, tg_access, dim_x)


class GenericImageAttr(ImageAttr):

    def __init__(self, name, tg_type, tg_access, dim_x=2048, dim_y=2048):
        ImageAttr.__init__(self, name, tg_type, tg_access, dim_x, dim_y)

def clean_device_attribute_memorized(db, dev_name, attr_name):
    props = "__value", "__value_ts"
    db.delete_device_attribute_property(dev_name, {attr_name : props })

def clean_device_memorized(db, dev_name):
    for attr_name in PyTango.DeviceProxy(dev_name).get_attribute_list():
        clean_device_attribute_memorized(db, dev_name, attr_name)

def clean_server_memorized(db, server_name, server_instance):
    server = server_name + "/" + server_instance
    dev_names = db.get_device_class_list(server)[::2]
    for dev_name in dev_names:
        clean_device_memorized(db, dev_name)

def __set_last_write_value(attribute, lrv):
    attribute._last_write_value = lrv
    return lrv

def __get_last_write_value(attribute):
    if hasattr(attribute, '_last_write_value'):
        lrv = attribute._last_write_value
    else:
        attribute._last_write_value = lrv = None
    return lrv

def memorize_write_attribute(write_attr_func):
    """The main purpose is to use this as a decorator for write_<attr_name>
       device methods.
       
       Properly memorizes the attribute write value:
           
           - only memorize if write doesn't throw exception
           - also memorize the timestamp
       
       :param write_attr_func: the write method 
       :type write_attr_func: callable
       :return: a write method safely wrapping the given write method
       :rtype: callable"""
       
    @wraps(write_attr_func)
    def write_attr_wrapper(self, attribute):
        ts = repr(time.time())
        attr_name = attribute.get_name()
        dev_name = self.get_name()

        if not isinstance(attribute, WAttribute):
            return write_attr_func(self, attribute)
        
        lwv = __get_last_write_value(attribute)
        wv = attribute.get_write_value(), ts
        store, raises_exc = wv, True
        store_value = False
        try:
            ret = write_attr_func(self, attribute)
            __set_last_write_value(attribute, wv)
            raises_exc = False
        finally:
            # if there is an exception recover from the last write value.
            # (don't catch and raise exceptions to avoid messing up the stack)
            if raises_exc:
                store_value = True
                if lwv is not None:
                    store = lwv
        if store is not None:
            db = self.get_database()
            attr_values = dict(__value_ts=store[1])
            if store_value:
                attr_values['__value'] = store[0]
            db.put_device_attribute_property(dev_name, { attr_name : attr_values })
        return ret
        
    return write_attr_wrapper
    
def tango_protect(wrapped, *args, **kwargs):
    @wraps(wrapped)
    def wrapper(self, *args, **kwargs):
        with self.tango_lock:
            return wrapped(self, *args, **kwargs)
    return wrapper

def from_deviceattribute_value(da):
    if da.has_failed:
        return
    dtype, dformat, value = da.type, da.data_format, da.value
    if dtype == PyTango.DevState:
        if dformat == PyTango.SCALAR:
            return from_tango_state_to_state(value)
        elif dformat == PyTango.SPECTRUM:
            return list(map(from_tango_state_to_state, value))
        elif dformat == PyTango.IMAGE:
            return [ list(map(from_tango_state_to_state, v)) for v in value ]
    return value

def from_deviceattribute(da):
    if da.has_failed:
        exc_info = DevFailed(*da.get_err_stack())
        value = None
    else:
        exc_info = None
        value = from_deviceattribute_value(da.value)
    
    dtype, dformat = from_tango_type_format(da.type, da.data_format) 
    
    ret = SardanaValue(value=value, exc_info=exc_info, 
                       timestamp=da.time.totime(), dtype=dtype, dformat=dformat)
    return ret

def to_tango_state(state):
    return DevState(state)

def from_tango_state_to_state(state):
    return int(state)

#: dictionary dict<:class:`sardana.DataType`, :class:`PyTango.CmdArgType`>
TTYPE_MAP = {
    DataType.Integer : DevLong,
    DataType.Double  : DevDouble,
    DataType.String  : DevString,
    DataType.Boolean : DevBoolean,
}
R_TTYPE_MAP = dict((v,k) for k,v in TTYPE_MAP.items())

#: dictionary dict<:class:`sardana.DataFormat`, :class:`PyTango.AttrFormat`>
TFORMAT_MAP = {
    DataFormat.Scalar : SCALAR,
    DataFormat.OneD   : SPECTRUM,
    DataFormat.TwoD   : IMAGE,
}
R_TFORMAT_MAP = dict((v,k) for k,v in TFORMAT_MAP.items())

#: dictionary dict<:class:`sardana.DataAccess`, :class:`PyTango.AttrWriteType`>
TACCESS_MAP = {
    DataAccess.ReadOnly  : READ,
    DataAccess.ReadWrite : READ_WRITE,
}

R_TACCESS_MAP = dict((v,k) for k,v in TACCESS_MAP.items())

def exception_str(etype=None, value=None, sep='\n'):
    if etype is None:
        etype, value = sys.exc_info()[:2]
    return sep.join(traceback.format_exception_only(etype, value))

def to_tango_access(access):
    """Transforms a :obj:`~sardana.DataAccess` into a
    :obj:`~PyTango.AttrWriteType`
    
    :param access: the access to be transformed
    :type access: :obj:`~sardana.DataAccess`
    :return: the tango attribute write type
    :rtype: :obj:`PyTango.AttrWriteType`"""
    return TACCESS_MAP[access]

def from_tango_access(access):
    """Transforms a :obj:`~PyTango.AttrWriteType` into a
    :obj:`~sardana.DataAccess`
    
    :param access: the tango access to be transformed
    :type access: :obj:`~PyTango.AttrWriteType`
    :return: the sardana attribute write type
    :rtype: :obj:`~sardana.DataAccess`"""
    return R_TACCESS_MAP[access]

def to_tango_type_format(dtype_or_info, dformat=None):
    """Transforms a :obj:`~sardana.DataType` :obj:`~sardana.DataFormat` into a
    :obj:`~PyTango.CmdArgType`, :obj:`~PyTango.AttrDataFormat` tuple
    
    :param dtype_or_info: the type to be transformed
    :type dtype_or_info: :obj:`~sardana.DataType`
    :param dformat: the format to be transformed
    :type dformat: :obj:`~sardana.DataFormat`
    
    :return: a tuple of two elements: the tango attribute write type, tango data format
    :rtype: tuple< :obj:`PyTango.CmdArgType`, :obj:`PyTango.AttrDataFormat` >"""
    dtype = dtype_or_info
    if dformat is None:
        dtype, dformat = to_dtype_dformat(dtype)
    return TTYPE_MAP.get(dtype, DevVoid), TFORMAT_MAP.get(dformat, FMT_UNKNOWN)

def from_tango_type_format(dtype, dformat=PyTango.SCALAR):
    """Transforms a :obj:`~PyTango.CmdArgType`, :obj:`~PyTango.AttrDataFormat`
    into a :obj:`~sardana.DataType` :obj:`~sardana.DataFormat` tuple
    
    :param dtype: the type to be transformed
    :type dtype: :obj:`~PyTango.CmdArgType`
    :param dformat: the format to be transformed
    :type dformat: :obj:`~PyTango.AttrDataFormat`
    
    :return: a tuple of two elements: data type, data format
    :rtype: tuple< :obj:`~sardana.DataType`, :obj:`~sardana.DataFormat` >"""
    return R_TTYPE_MAP[dtype], R_TFORMAT_MAP[dformat]

def to_tango_attr_info(attr_name, attr_info):
    if isinstance(attr_info, DataInfo):
        data_type, data_format = attr_info.dtype, attr_info.dformat
        data_access = attr_info.access
        desc = attr_info.description
        memorized = attr_info.memorized
    else:
        data_type, data_format = to_dtype_dformat(attr_info.get('type'))
        data_access = to_daccess(attr_info.get('r/w type'))
        desc = attr_info.get('description')
        memorized = attr_info.get('memorized')
    
    tg_type, tg_format = to_tango_type_format(data_type, data_format)
    tg_access = to_tango_access(data_access)
    tg_attr_info = [ [ tg_type, tg_format, tg_access ] ]

    extra = {}
    tg_attr_info.append(extra)

    if desc is not None and len(desc) > 0:
        extra['description'] = desc
    extra['memorized'] = memorized
    return attr_name, tg_attr_info

def throw_sardana_exception(exc):
    """Throws an exception as a tango exception"""
    if isinstance(exc, SardanaException):
        if exc.exc_info and not None in exc.exc_info:
            Except.throw_python_exception(*exc.exc_info)
        else:
            tb = "<Unknown>"
            if exc.traceback is not None:
                tb = str(exc.traceback)
            Except.throw_exception(exc.msg, tb, exc.type)
    elif hasattr(exc, 'exc_info'):
        Except.throw_python_exception(*exc.exc_info)
        
def ask_yes_no(prompt,default=None):
    """Asks a question and returns a boolean (y/n) answer.

    If default is given (one of 'y','n'), it is used if the user input is
    empty. Otherwise the question is repeated until an answer is given.

    An EOF is treated as the default answer.  If there is no default, an
    exception is raised to prevent infinite loops.

    Valid answers are: y/yes/n/no (match is not case sensitive)."""
    answers = {'y':True,'n':False,'yes':True,'no':False}
    ans = None
    if default is not None:
        d_l = default.lower()
        if d_l in ('y','yes'):
            prompt += " (Y/n) ?"
        elif d_l in ('n','no'):
            prompt += " (N/y) ?"
    
    while ans not in answers.keys():
        try:
            ans = raw_input(prompt+' ').lower()
            if not ans:  # response was an empty string
                ans = default
        except KeyboardInterrupt:
            print
        except EOFError:
            if default in answers.keys():
                ans = default
                print
            else:
                raise

    return answers[ans]
    
def clean_tango_args(args):
    ret, ret_for_tango, ret_for_ORB = [], [], []
    
    tango_args = "-?", "-nodb", "-file="
    nb_args = len(args)
    i = 0
    while i < nb_args:
        arg = args[i]
        try:
            if arg.startswith("-v") and int(arg[2:]):
                ret_for_tango.append(arg)
                i += 1
                continue
        except:
            pass
        if arg.startswith('-ORB'):
            ret_for_ORB.append(arg)
            ret_for_tango.append(arg)
            i += 1
            if i < nb_args:
                ret_for_ORB.append(args[i])
                ret_for_tango.append(args[i])
                i += 1
            continue
        if arg.startswith(tango_args):
            ret_for_tango.append(arg)
            i += 1
            continue
        if arg == "-dlist":
            ret_for_tango.append(arg)
            i += 1
            while i < nb_args and args[i][0] != "-":
                arg = args[i]
                ret_for_tango.append(arg)
                i += 1
            continue
        ret.append(arg)
        i += 1
    return ret, ret_for_tango, ret_for_ORB
        
def prepare_cmdline(parser=None, args=None):
    """Prepares the command line separating tango options from server specific
    options.
    
    :return: a sequence of options, arguments, tango arguments
    :rtype: seq<opt, list<str>, list<str>>"""
    import optparse
    if args is None:
        args = []
    
    proc_args, tango_args, ORB_args = clean_tango_args(args)
    
    if parser is None:
        version = "%s" % (Release.version)
        parser = optparse.OptionParser(version=version)
    
    parser.usage = "usage: %prog instance_name [options]"
    log_level_choices = "critical", "error", "warning", "info", "debug", "trace", \
                        "0", "1", "2", "3", "4", "5"
    help_olog = "log output level. Possible values are (case sensitive): " \
                "critical (or 0), error (1), warning (2), info (3) " \
                "debug (4), trace (5) [default: %default]"
    help_flog = "log file level. Possible values are (case sensitive): " \
                "critical (or 0), error (1), warning (2), info (3) " \
                "debug (4), trace (5) [default: %default]. " \
                "Ignored if --without-log-file is True"
    help_fnlog = "log file name. When given, MUST be absolute file name. " \
                "[default: /tmp/tango/<DS name>/<DS instance name lower case>/log.txt]. " \
                "Ignored if --without-log-file is True"
    help_wflog = "When set to True disables logging into a file [default: %default]"
    help_rfoo = "rconsole port number. [default: %default meaning rconsole NOT active]"
    parser.add_option("--log-level", dest="log_level", metavar="LOG_LEVEL",
                      help=help_olog, type="choice", choices=log_level_choices, default="warning")
    parser.add_option("--log-file-level", dest="log_file_level", metavar="LOG_FILE_LEVEL",
                      help=help_flog, type="choice", choices=log_level_choices, default="debug")
    parser.add_option("--log-file-name", dest="log_file_name",
                      help=help_fnlog, type="str", default=None)
    parser.add_option("--without-log-file", dest="without_log_file",
                      help=help_wflog, default=False)
    
    parser.add_option("--rconsole-port", dest="rconsole_port",
                      metavar="RCONSOLE_PORT", help=help_rfoo, type="int", default=0)

    res = list( parser.parse_args(proc_args) )
    tango_args = res[1][:2] + tango_args
    res.append(tango_args)
    res.append(ORB_args)
    return res

def prepare_environment(args, tango_args, ORB_args):
    """Since we have to create a Tango Database object before the Tango Util,
    omniORB doesn't recognize parameters on the command line anymore
    (tango, omniORB bug?), so we export these parameters as environment
    variables (this workaround seems to work)"""
    log_messages = []
    ORB_args_len = len(ORB_args)
    for i in range(ORB_args_len):
        arg = ORB_args[i]
        if arg.startswith("-ORB") and i+1 < ORB_args_len:
            env_name = arg[1:]
            env_val = ORB_args[i+1]
            os.environ[env_name] = env_val
            log_messages.append(("setting %s=%s", env_name, env_val))
    return log_messages

def prepare_server(args, tango_args):
    """Register a proper server if the user gave an unknown server"""
    log_messages = []
    _, bin_name = os.path.split(args[0])
    
    if "-?" in tango_args:
        return log_messages
    
    nodb = "-nodb" in tango_args
    if nodb and not hasattr(DeviceClass, "device_name_factory"):
        print "In order to start %s with 'nodb' you need PyTango >= 7.2.3" % bin_name
        sys.exit(1)
    
    if len(tango_args) < 2:
        valid = False
        while not valid:
            inst_name = raw_input("Please indicate %s instance name: " % bin_name)
            #should be a instance name validator.
            valid_set=string.letters + string.digits + '_' + '-'
            out=''.join([c for c in inst_name if c not in valid_set])
            valid = len(inst_name) > 0 and len(out)==0
            if not valid:
                print "We only accept alphanumeric combinations"
        args.append(inst_name)
        tango_args.append(inst_name)
    else:
        inst_name = tango_args[1].lower()
    
    if "-nodb" in tango_args:
        return log_messages
    
    db = Database()
    if not exists_server_instance(db, bin_name, inst_name):
        if ask_yes_no('%s does not exist. Do you wish create a new one' % inst_name, default='y'):
            if bin_name == 'MacroServer' :
                # build list of pools to which the MacroServer should connect to
                pool_names = []
                pools = get_dev_from_class(db, "Pool")
                all_pools = pools.keys()
                for pool in pools.values():
                    pool_alias = pool[2]
                    if pool_alias is not None:
                        all_pools.append(pool_alias)
                all_pools = map(str.lower, all_pools)
                for i in pools:
                    print pools[i][3]
                while True:
                    elem = raw_input("Please select pool to connect to (return to finish): ").strip()
                    if not len(elem):
                        break
                    if elem.lower() not in all_pools:
                        print "Unknown pool element"
                        print all_pools
                    else:
                        pool_names.append(elem)
                    log_messages += register_sardana(db, bin_name, inst_name, pool_names)
            else:
                log_messages += register_sardana(db, bin_name, inst_name)
    return log_messages
    
def exists_server_instance(db, server_name, server_instance):
    known_inst = map(str.lower, db.get_instance_name_list(server_name))
    return server_instance.lower() in known_inst

def register_sardana(db, bin_name, inst_name, pool_names=None):
    devices = []
    log_messages = []
    if bin_name == 'MacroServer':
        props = {'PoolNames' : pool_names}
        ms_alias = get_free_alias(db, "MS_" + inst_name)
        devices.append(('MacroServer', None, ms_alias, props))
        door_alias = get_free_alias(db, "Door_" + inst_name)
        devices.append(("Door", None, door_alias, {}))                
    elif bin_name == 'Pool':
        pool_alias = get_free_alias(db, 'Pool_' + inst_name)
        devices.append(('Pool', None, pool_alias, {}))
    elif bin_name == 'Sardana':
        pool_dev_name = get_free_device(db, 'pool/' + inst_name)
        pool_alias = get_free_alias(db, 'Pool_' + inst_name)
        devices.append(('Pool', pool_dev_name, pool_alias, {}))
        ms_alias = get_free_alias(db, "MS_" + inst_name)
        devices.append(('MacroServer', None, ms_alias,
                        {'PoolNames' : [pool_dev_name]}))
        door_alias = get_free_alias(db, "Door_" + inst_name)
        devices.append(("Door", None, door_alias, {}))
    register_server_with_devices(db, bin_name, inst_name, devices)
    log_messages.append(("Registered server '%s/%s'", bin_name, inst_name))
    for d in devices:
        dev_class, dev_alias = d[0], d[2]
        log_messages.append(("Registered %s %s", dev_class, dev_alias))
    return log_messages

def register_server_with_devices(db, server_name, server_instance, devices):
    """Registers a new server with some devices in the Database.
       Devices is a seq<tuple<str, str, str, dict>>> where each item is a
       sequence of 4 elements :
        - device class
        - device name prefix
        - device alias
        - dictionary of properties

       :param db: database where to register devices
       :type db: PyTango.Database
       :param server_name: server name
       :type server_name: str
       :param server_instance: server instance name
       :type server_instance: str
       :param devices: map of devices to create. 
       :type devices: dict<str, seq<tuple<str, str, dict>>>
    """
    info = DbDevInfo()
    info.server = server_name + "/" + server_instance
    for dev_info in devices:
        dev_class, prefix, alias, props = dev_info
        if prefix is None:
            prefix = dev_class + "/" + server_instance
        if prefix.count("/") == 1:
            prefix = get_free_device(db, prefix)
        info._class = dev_class
        info.name   = prefix
        db.add_device(info)
        if alias is None:
            alias_prefix = dev_class + "_" + server_instance
            alias = get_free_alias(db, alias_prefix)
        db.put_device_alias(info.name, alias)
        if props is not None:
            db.put_device_property(info.name, props)

def from_name_to_tango(db, name):
    alias = None
    
    c = name.count('/')
    # if the db prefix is there, remove it first
    if c == 3 or c == 1:
        name = name[name.index("/")+1:]
    
    elems = name.split('/')
    l = len(elems)
    
    if l == 3:
        try:
            alias = db.get_alias(name)
            if alias.lower() == 'nada':
                alias = None
        except:
            alias = None
    elif l == 1:
        alias = name
        name = db.get_device_alias(alias)
    else:
        raise Exception("Invalid device name '%s'" % name)
    
    full_name = "%s:%s/%s" % (db.get_db_host(), db.get_db_port(), name)
    return full_name, name, alias

def get_dev_from_class(db, classname):
    """Returns tuple<full device name, device name, alias, ouput string>"""
    server_wildcard = '*'
    try:
        exp_dev_list = db.get_device_exported_for_class(classname)
    except Exception: 
        exp_dev_list = []
    
    res = {}
    dev_list = db.get_device_name(server_wildcard, classname)
    for dev in dev_list:
        full_name, name, alias = from_name_to_tango(db, dev)
        out = alias or name
        if alias: out += ' (a.k.a. %s)' % name
        out = "%-25s" % out 
        if dev in exp_dev_list:
            out += " (running)"
        res[dev] = full_name, name, alias, out
    return res

def get_free_device(db, prefix, start_from=1):
    members = db.get_device_member(prefix + "/*")
    while str(start_from) in members:
        start_from += 1
    return prefix + "/" + str(start_from)

def get_free_alias(db, prefix, start_from=1):
    while True:
        name = prefix + "_" + str(start_from)
        try:
            db.get_alias(name)
            start_from += 1
        except:
            return name

def prepare_taurus(options, args, tango_args):
    # make sure the polling is not active
    factory = taurus.Factory()
    factory.disablePolling()
    
def prepare_logging(options, args, tango_args, start_time=None, log_messages=None):
    taurus.setLogLevel(taurus.Debug)
    root = Logger.getRootLog()
    
    # output logger configuration
    log_output_level = options.log_level
    log_level_map  = { "0" : taurus.Critical, "critical" : taurus.Critical,
                       "1" : taurus.Error, "error" : taurus.Error,
                       "2" : taurus.Warning, "warning" : taurus.Warning,
                       "3" : taurus.Info, "info" : taurus.Info,
                       "4" : taurus.Debug, "debug" : taurus.Debug,
                       "5" : taurus.Trace, "trace" : taurus.Trace,
                     }
    log_output_level = log_level_map[log_output_level]
    root.handlers[0].setLevel(log_output_level)
    
    if not options.without_log_file:
        log_file_level = options.log_file_level
        log_file_level = log_level_map[log_file_level]
        
        # Create a file handler
        if options.log_file_name is None:
            _, ds_name = os.path.split(args[0])
            ds_name, _ = os.path.splitext(ds_name)
            ds_instance = args[-1].lower()
            import getpass
            try:
                tangodir = 'tango-%s'%getpass.getuser() #include the user name to avoid permission errors
            except:
                tangodir = 'tango'%getpass.getuser()
            path = os.path.join(os.sep, "tmp", tangodir, ds_name, ds_instance)
            log_file_name = os.path.join(path, 'log.txt')
        else:
            log_file_name = options.log_file_name
        path = os.path.dirname(log_file_name)
        
        # because some versions of python have a bug in logging.shutdown (this
        # function is not protected against deleted handlers) we store the
        # handlers we create to make sure a strong reference exists when the
        # logging.shutdown is called
        taurus._handlers = handlers = []
        try:
            if not os.path.exists(path):
                os.makedirs(path, 0777)
            
            fmt = Logger.getLogFormat()
            f_h = logging.handlers.RotatingFileHandler(log_file_name,
                                                       maxBytes=1E7,
                                                       backupCount=5)
            f_h.setFormatter(fmt)
            f_h.setLevel(log_file_level)
            root.addHandler(f_h)
            handlers.append(f_h)
            
            if start_time is not None:
                taurus.info("Started at %s", start_time)
            else:
                taurus.info("Starting up...")
            taurus.info("Log is being stored in %s", log_file_name)
        except:
            if start_time is not None:
                taurus.info("Started at %s", start_time)
            else:
                taurus.info("Starting up...")
            taurus.warning("'%s' could not be created. Logs will not be stored",
                           log_file_name)
            taurus.debug("Error description", exc_info=1)
    
    if log_messages is None:
        log_messages = []
    for log_message in log_messages:
        taurus.info(*log_message)

    taurus.debug("Start args=%s", args)
    taurus.debug("Start tango args=%s", tango_args)
    taurus.debug("Start options=%s", options)
    taurus.debug("Using PyTango %s from %s", PyTango.Release.version, PyTango.__path__[0])
    taurus.debug("Using taurus %s from %s", taurus.Release.version, taurus.__path__[0])
    taurus.debug("Using sardana %s from %s", sardana.Release.version, sardana.__path__[0])

def prepare_rconsole(options, args, tango_args):
    port = options.rconsole_port
    if port is None or port is 0:
        return
    taurus.debug("Setting up rconsole on port %d...", port)
    try:
        import rfoo.utils.rconsole
        rfoo.utils.rconsole.spawn_server(port=port)
        taurus.debug("Finished setting up rconsole")
    except:
        taurus.debug("Failed to setup rconsole", exc_info=1)

def run_tango_server(tango_util=None, start_time=None):
    try:
        if tango_util is None:
            tango_util = Util(sys.argv)
        util = Util.instance()
        SardanaServer.server_state = State.Init
        util.server_init()
        SardanaServer.server_state = State.Running
        if start_time is not None:
            import datetime
            dt = datetime.datetime.now() - start_time
            taurus.info("Ready to accept request in %s", dt)
        else:
            taurus.info("Ready to accept request")
        util.server_run()
        SardanaServer.server_state = State.Off
        taurus.info("Exiting")
    except DevFailed:
        taurus.info("Exiting")
        taurus.critical("Server exited with DevFailed", exc_info=1)
    except KeyboardInterrupt:
        taurus.info("Exiting")
        taurus.critical("Interrupted by keyboard")
    except Exception:
        taurus.info("Exiting")
        taurus.critical("Server exited with unforeseen exception", exc_info=1)
    taurus.info("Exited")

def run(prepare_func, args=None, tango_util=None, start_time=None, mode=None):
    
    if mode is None:
        mode = ServerRunMode.SynchPure
        
    if args is None:
        if mode != ServerRunMode.SynchPure:
            raise Exception("When running in separate thread/process, " \
                            "'args' must be given")
        args = sys.argv
    
    name = args[0]
    
    if mode != ServerRunMode.SynchPure:
        if mode in (ServerRunMode.SynchThread, ServerRunMode.AsynchThread):
            import threading
            class task_klass(threading.Thread):
                def terminate(self):
                    if not self.is_alive():
                        return
                    Util.instance().get_dserver_device().kill()
        else:
            import multiprocessing
            task_klass = multiprocessing.Process
            tango_util = None

        task_args = prepare_func,
        task_kwargs = dict(args=args, tango_util=tango_util,
                           start_time=start_time, mode=ServerRunMode.SynchPure)
        
        task = task_klass(name=name, target=run, args=task_args,
                          kwargs=task_kwargs)
        task.daemon = False
        task.start()
        if mode in (ServerRunMode.SynchThread, ServerRunMode.SynchProcess):
            task.join()
        return task
    
    log_messages = []
    try:
        options, args, tango_args, ORB_args = prepare_cmdline(args=args)
    except KeyboardInterrupt:
        pass
    
    log_messages.extend(prepare_environment(args, tango_args, ORB_args))
    log_messages.extend(prepare_server(args, tango_args))

    if tango_util == None:
        tango_util = Util(tango_args)
    
    prepare_func(tango_util)
    prepare_taurus(options, args, tango_args)
    prepare_logging(options, args, tango_args, start_time=start_time,
                    log_messages=log_messages)
    prepare_rconsole(options, args, tango_args)
    
    run_tango_server(tango_util, start_time=start_time)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

##############################################################################
##
## This file is part of Sardana
## 
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""reST directive for syntax-highlighting spock interactive sessions.
"""

#-----------------------------------------------------------------------------
# Needed modules

# Standard library
import re
import copy

# Third party
from pygments.lexer import Lexer, do_insertions
from pygments.lexers.agile import (PythonConsoleLexer, PythonLexer, 
                                   PythonTracebackLexer)
from pygments.token import Token, Comment, Generic
from pygments.style import Style
import pygments.styles
from sphinx import highlighting

#-----------------------------------------------------------------------------
# Global constants
line_re = re.compile('.*?\n')

DftStyle = pygments.styles.get_style_by_name("default")

class SpockStyle(DftStyle):
    
    styles = copy.copy(DftStyle.styles)
    styles[Generic.Prompt] = 'bold #00AA00'

class SpockConsoleLexer(Lexer):
    """
    For spock console output or doctests, such as:

    .. sourcecode:: spock

      LAB-01 [1]: a = 'foo'

      LAB-01 [2]: a
      Result [2]: 'foo'

      LAB-01 [3]: print a
      foo

      LAB-01 [4]: 1 / 0

    Notes:

      - Tracebacks are not currently supported.

      - It assumes the default spock prompts, not customized ones.
    """
    
    name = 'Spock console session'
    aliases = ['spock']
    mimetypes = ['text/x-spock-console']
    input_prompt = re.compile("([a-zA-Z0-9_\-]+ \[(?P<N>[0-9]+)\]: )|(   \.\.\.+:)")
    output_prompt = re.compile("(\s*Result \[(?P<N>[0-9]+)\]: )|(   \.\.\.+:)")
    continue_prompt = re.compile("   \.\.\.+:")
    tb_start = re.compile("\-+")

    def get_tokens_unprocessed(self, text):
        pylexer = PythonLexer(**self.options)
        tblexer = PythonTracebackLexer(**self.options)

        curcode = ''
        insertions = []
        for match in line_re.finditer(text):
            line = match.group()
            input_prompt = self.input_prompt.match(line)
            continue_prompt = self.continue_prompt.match(line.rstrip())
            output_prompt = self.output_prompt.match(line)
            if line.startswith("#"):
                insertions.append((len(curcode),
                                   [(0, Comment, line)]))
            elif line.startswith("<warning>"):
                insertions.append((len(curcode),
                                   [(0, Generic.Error, line[9:])]))
            elif input_prompt is not None:
                insertions.append((len(curcode),
                                   [(0, Generic.Prompt, input_prompt.group())]))
                curcode += line[input_prompt.end():]
            elif continue_prompt is not None:
                insertions.append((len(curcode),
                                   [(0, Generic.Prompt, continue_prompt.group())]))
                curcode += line[continue_prompt.end():]
            elif output_prompt is not None:
                # Use the 'error' token for output.  We should probably make
                # our own token, but error is typicaly in a bright color like
                # red, so it works fine for our output prompts.
                insertions.append((len(curcode),
                                   [(0, Generic.Error, output_prompt.group())]))
                curcode += line[output_prompt.end():]
            else:
                if curcode:
                    for item in do_insertions(insertions,
                                              pylexer.get_tokens_unprocessed(curcode)):
                        yield item
                        curcode = ''
                        insertions = []
                yield match.start(), Generic.Output, line
        if curcode:
            for item in do_insertions(insertions,
                                      pylexer.get_tokens_unprocessed(curcode)):
                yield item


def setup(app):
    """Setup as a sphinx extension."""

    # This is only a lexer, so adding it below to pygments appears sufficient.
    # But if somebody knows that the right API usage should be to do that via
    # sphinx, by all means fix it here.  At least having this setup.py
    # suppresses the sphinx warning we'd get without it.
    pass

#-----------------------------------------------------------------------------
# Register the extension as a valid pygments lexer
highlighting.lexers['spock'] = SpockConsoleLexer()



##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module contains the definition of a slit pseudo motor controller
for the Sardana Device Pool"""

__all__ = ["Slit"]

__docformat__ = 'restructuredtext'

from sardana import DataAccess
from sardana.pool.controller import PseudoMotorController
from sardana.pool.controller import DefaultValue, Description, Access, Type

    
class Slit(PseudoMotorController):
    """A Slit pseudo motor controller for handling gap and offset pseudo 
       motors. The system uses to real motors sl2t (top slit) and sl2b (bottom
       slit)"""
    
    gender = "Slit"
    model  = "Default Slit"
    organization = "Sardana team"
    
    pseudo_motor_roles = "Gap", "Offset"
    motor_roles = "sl2t", "sl2b"
    
    ctrl_properties = { 'sign' : { Type : float,
                                   Description  : 'Gap = sign * calculated gap\nOffset = sign * calculated offet',
                                   DefaultValue : 1 }, }
    
    axis_attributes = { 'example' : { Type : int,
                                      Access : DataAccess.ReadWrite,
                                      Description : 'test purposes' }, }

    def __init__(self, inst, props, *args, **kwargs):
        PseudoMotorController.__init__(self, inst, props, *args, **kwargs)
        self._log.debug("Created SLIT %s", inst)
        self._example = {}
        
    def CalcPhysical(self, index, pseudo_pos, curr_physical_pos):
        half_gap = pseudo_pos[0]/2.0
        if index == 1:
            ret = self.sign * (pseudo_pos[1] + half_gap)
        else:
            ret = self.sign * (half_gap - pseudo_pos[1])
        self._log.debug("Slit.CalcPhysical(%d, %s) -> %f", index, pseudo_pos, ret)
        return ret
    
    def CalcPseudo(self, index, physical_pos, curr_pseudo_pos):
        gap = physical_pos[1] + physical_pos[0]
        if index == 1:
            ret = self.sign * gap
        else:
            ret = self.sign * (physical_pos[0] - gap/2.0)
        return ret
        
    def CalcAllPseudo(self, physical_pos, curr_pseudo_pos):
        """Calculates the positions of all pseudo motors that belong to the 
           pseudo motor system from the positions of the physical motors."""
        gap = physical_pos[1] + physical_pos[0]
        return (self.sign * gap,
                self.sign * (physical_pos[0] - gap/2.0))
    
    #def CalcAllPhysical(self, pseudo_pos, curr_physical_pos):
    #    """Calculates the positions of all motors that belong to the pseudo 
    #       motor system from the positions of the pseudo motors."""
    #    half_gap = pseudo_pos[0]/2.0
    #    return (self.sign * (pseudo_pos[1] + half_gap),
    #            self.sign * (half_gap - pseudo_pos[1]))

    def SetAxisExtraPar(self, axis, parameter, value):
        self._example[axis] = value
    
    def GetAxisExtraPar(self, axis, parameter):
        return self._example.get(axis, -1)



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module is part of the Python Sardana libray. It defines the base
classes for MetaLibrary and MetaClass"""

__all__ = ["SardanaLibrary", "SardanaClass", "SardanaFunction"]

__docformat__ = 'restructuredtext'

import os
import inspect
import linecache
import string
import weakref
import traceback

from sardanabase import SardanaBaseObject

# ----------------------------------------------------------------------------
# Start patch around inspect issue http://bugs.python.org/issue993580

def findsource(obj):
    """Return the entire source file and starting line number for an object.

    The argument may be a module, class, method, function, traceback, frame,
    or code object.  The source code is returned as a list of all the lines
    in the file and the line number indexes a line in that list.  An IOError
    is raised if the source code cannot be retrieved."""
    filename = inspect.getsourcefile(obj)
    if filename:
        linecache.checkcache(filename)
    return inspect.findsource(obj)

def getsourcelines(object):
    """Return a list of source lines and starting line number for an object.

    The argument may be a module, class, method, function, traceback, frame,
    or code object.  The source code is returned as a list of the lines
    corresponding to the object and the line number indicates where in the
    original source file the first line of code was found.  An IOError is
    raised if the source code cannot be retrieved."""
    lines, lnum = findsource(object)

    if inspect.ismodule(object): return lines, 0
    else: return inspect.getblock(lines[lnum:]), lnum + 1

def getsource(object):
    """Return the text of the source code for an object.

    The argument may be a module, class, method, function, traceback, frame,
    or code object.  The source code is returned as a single string.  An
    IOError is raised if the source code cannot be retrieved."""
    lines, lnum = getsourcelines(object)
    return string.join(lines, '')

# End patch around inspect issue http://bugs.python.org/issue993580
# ----------------------------------------------------------------------------


class SardanaLibrary(SardanaBaseObject):
    """Object representing a python module containing sardana classes.
       Public members:

           - module - reference to python module
           - file_path - complete (absolute) path (with file name at the end)
           - file_name - file name (including file extension)
           - path - complete (absolute) path
           - name - (=module name) module name (without file extension)
           - meta_classes - dict<str, SardanMetaClass>
           - exc_info - exception information if an error occurred when loading
                        the module"""

    description = '<Undocumented>'

    def __init__(self, **kwargs):
        self.module = module = kwargs.pop('module', None)
        self.file_path = file_path = kwargs.pop('file_path', None)
        self.exc_info = kwargs.pop('exc_info', None)
        if module is not None:
            file_path = os.path.abspath(module.__file__)
        self.file_path = file_path
        if file_path is None:
            self.path = kwargs.get('path', None)
            self.file_name = kwargs.get('file_name', None)
            name = kwargs.get('name', None)
        else:
            if self.file_path.endswith(".pyc"):
                self.file_path = self.file_path[:-1]
            self.path, self.file_name = os.path.split(self.file_path)
            name, _ = os.path.splitext(self.file_name)
        self.meta_classes = {}
        self.meta_functions = {}
        if module is not None and module.__doc__:
            self.description = module.__doc__
            self._code = getsourcelines(module)[0]
        else:
            self.description = name + " in error!"
            self._code = None
        kwargs['name'] = name
        kwargs['full_name'] = file_path or name
        SardanaBaseObject.__init__(self, **kwargs)

    def __cmp__(self, o):
        return cmp(self.full_name, o.full_name)

    def __str__(self):
        return self.name

    @property
    def module_name(self):
        """Returns the module name for this library.

        :return: the module name
        :rtype: str"""
        return self.name

    @property
    def code(self):
        """Returns a sequence of sourcelines corresponding to the module code.

           :return: list of source code lines
           :rtype: list<str>"""
        code = self._code
        if code is None:
            raise IOError('source code not available')
        return code

    def add_meta_class(self, meta_class):
        """Adds a new :class:~`sardana.sardanameta.SardanaClass` to this
        library.

        :param meta_class: the meta class to be added to this library
        :type meta_class: :class:~`sardana.sardanameta.SardanaClass`"""
        self.meta_classes[meta_class.name] = meta_class

    def get_meta_class(self, meta_class_name):
        """Returns a :class:~`sardana.sardanameta.SardanaClass` for the
        given meta class name or None if the meta class does not exist in this
        library.

        :param meta_class_name: the meta class name
        :type meta_class_name: str
        :return: a meta class or None
        :rtype: :class:~`sardana.sardanameta.SardanaClass`"""
        return self.meta_classes.get(meta_class_name)

    def get_meta_classes(self):
        """Returns a sequence of the meta classes that belong to this library.

        :return: a sequence of meta classes that belong to this library
        :rtype: seq<:class:~`sardana.sardanameta.SardanaClass`>"""
        return self.meta_classes.values()

    def has_meta_class(self, meta_class_name):
        """Returns True if the given meta class name belongs to this library
        or False otherwise.

        :param meta_class_name: the meta class name
        :type meta_class_name: str

        :return: True if the given meta class name belongs to this library
                 or False otherwise
        :rtype: bool"""
        return meta_class_name in self.meta_classes

    def add_meta_function(self, meta_function):
        """Adds a new :class:~`sardana.sardanameta.SardanaFunction` to this
        library.

        :param meta_function: the meta function to be added to this library
        :type meta_function: :class:~`sardana.sardanameta.SardanaFunction`"""
        self.meta_functions[meta_function.name] = meta_function

    def get_meta_function(self, meta_function_name):
        """Returns a :class:~`sardana.sardanameta.SardanaFunction` for the
        given meta function name or None if the meta function does not exist in
        this library.

        :param meta_function_name: the meta function name
        :type meta_function_name: str
        :return: a meta function or None
        :rtype: :class:~`sardana.sardanameta.SardanaFunction`"""
        return self.meta_functions.get(meta_function_name)

    def get_meta_functions(self):
        """Returns a sequence of the meta functions that belong to this library.

        :return: a sequence of meta functions that belong to this library
        :rtype: seq<:class:~`sardana.sardanameta.SardanaFunction`>"""
        return self.meta_functions.values()

    def has_meta_function(self, meta_function_name):
        """Returns True if the given meta function name belongs to this library
        or False otherwise.

        :param meta_function_name: the meta function name
        :type meta_function_name: str

        :return: True if the given meta function name belongs to this library
                 or False otherwise
        :rtype: bool"""
        return meta_function_name in self.meta_functions

    def get_meta(self, meta_name):
        """Returns a :class:~`sardana.sardanameta.SardanaCode` for the
        given meta name or None if the meta does not exist in this library.

        :param meta_name: the meta name (class, function)
        :type meta_name: str
        :return: a meta or None
        :rtype: :class:~`sardana.sardanameta.SardanaCode`"""
        ret = self.get_meta_class(meta_name)
        if ret is None:
            ret = self.get_meta_function(meta_name)
        return ret

    def has_meta(self, meta_name):
        """Returns True if the given meta name belongs to this library
        or False otherwise.

        :param meta_name: the meta name
        :type meta_name: str

        :return:
            True if the given meta (class or function) name belongs to this
            library or False otherwise
        :rtype: bool"""
        return self.has_meta_class(meta_name) or \
               self.has_meta_function(meta_name)

    def get_metas(self):
        """Returns a sequence of the meta (class and functions) that belong to
        this library.

        :return:
            a sequence of meta (class and functions) that belong to this library
        :rtype: seq<:class:~`sardana.sardanameta.SardanaCode`>"""
        return self.get_meta_classes() + self.get_meta_functions()

    def get_name(self):
        """Returns the module name for this library (same as
        :meth:~sardana.sardanameta.SardanaLibrary.get_module_name).

        :return: the module name
        :rtype: str"""
        return self.name

    def get_module_name(self):
        """Returns the module name for this library (same as
        :meth:~sardana.sardanameta.SardanaLibrary.get_name).

        :return: the module name
        :rtype: str"""
        return self.module_name

    def get_module(self):
        """Returns the python module for this library.

        :return: the python module
        :rtype: object"""
        return self.module

    def get_description(self):
        """Returns the this library documentation or "<Undocumented>" if no
        documentation exists.

        :return: this library documentation or None
        :rtype: str"""
        return self.description

    def get_code(self):
        """Returns a sequence of sourcelines corresponding to the module code.

       :return: list of source code lines
       :rtype: list<str>"""
        return self.code

    def get_file_path(self):
        """Returns the file path for this library. On posix systems is something
        like: /abs/path/filename.py

        :return: this library file path
        :rtype: str"""
        if self.file_path is None:
            return None
        if self.file_path.endswith('.pyc'):
            return self.file_path[:-1]
        return self.file_path

    def get_file_name(self):
        """Returns the file name for this library. On posix systems is something
        like: filename.py

        :return: this library file name
        :rtype: str"""
        return self.file_name

    def has_errors(self):
        """Returns True if this library has syntax errors or False otherwise.

        :return: True if this library has syntax errors or False otherwise
        :rtype: bool"""
        return self.exc_info != None

    def set_error(self, exc_info):
        """Sets the error information for this library

        :param exc_info: error information. It must be an object similar to the
                         one returned by :func:`sys.exc_info`
        :type exc_info: tuple<type, value, traceback>"""
        self.exc_info = exc_info
        if exc_info is None:
            self.meta_classes = {}
            self.meta_functions = {}

    def get_error(self):
        """Gets the error information for this library or None if no error
        exists

        :return: error information. An object similar to the one returned by
                 :func:`sys.exc_info`
        :rtype: tuple<type, value, traceback>"""
        return self.exc_info

    def serialize(self, *args, **kwargs):
        """Returns a serializable object describing this object.

        :return: a serializable dict
        :rtype: dict"""
        kwargs = SardanaBaseObject.serialize(self, *args, **kwargs)
        kwargs['id'] = 0
        kwargs['module'] = self.name
        kwargs['file_path'] = self.file_path
        kwargs['file_name'] = self.file_name
        kwargs['path'] = self.path
        kwargs['description'] = self.description
        kwargs['elements'] = self.meta_classes.keys() + self.meta_functions.keys()
        if self.exc_info is None:
            kwargs['exc_summary'] = None
            kwargs['exc_info'] = None
        else:
            kwargs['exc_summary'] = "".join(traceback.format_exception_only(*self.exc_info[:2]))
            kwargs['exc_info'] = "".join(traceback.format_exception(*self.exc_info))
        return kwargs


class SardanaCode(SardanaBaseObject):
    """Object representing a python code (base for class and function)."""

    description = '<Undocumented>'

    def __init__(self, **kwargs):
        lib = kwargs.pop('lib')
        self._lib = weakref.ref(lib)
        self._code_obj = kwargs.pop('code')
        doc = self._code_obj.__doc__
        if doc:
            self.description = doc
        self._code = getsourcelines(self._code_obj)
        name = kwargs['name']
        kwargs['full_name'] = "{0}.{1}".format(lib.name, name)
        kwargs['parent'] = kwargs.pop('parent', self.lib)
        SardanaBaseObject.__init__(self, **kwargs)

    @property
    def code_object(self):
        return self._code_obj

    @property
    def lib(self):
        """Returns the library :class:~`sardana.sardanameta.SardanaLibrary`
        for this class.

        :return: a reference to the library where this class is located
        :rtype: :class:~`sardana.sardanameta.SardanaLibrary`"""
        return self._lib()

    @property
    def module(self):
        """Returns the python module for this class.

        :return: the python module
        :rtype: object"""
        return self.lib.module

    @property
    def module_name(self):
        """Returns the module name for this class.

        :return: the module name
        :rtype: str"""
        return self.lib.get_module_name()

    @property
    def file_path(self):
        """Returns the file path for for the library where this class is. On
        posix systems is something like: /abs/path/filename.py

        :return: the file path for for the library where this class is
        :rtype: str"""
        return self.lib.file_path

    @property
    def file_name(self):
        """Returns the file name for the library where this class is. On posix
        systems is something like: filename.py

        :return: the file name for the library where this class is
        :rtype: str"""
        return self.lib.file_name

    @property
    def path(self):
        """Returns the absolute path for the library where this class is. On
        posix systems is something like: /abs/path

        :return: the absolute path for the library where this class is
        :rtype: str"""
        return self.lib.path

    @property
    def code(self):
        """Returns a tuple (sourcelines, firstline) corresponding to the
        definition of this code object. sourcelines is a list of source code
        lines. firstline is the line number of the first source code line."""
        code = self._code
        if code is None:
            raise IOError('source code not available')
        return code

    def get_code(self):
        """Returns a tuple (sourcelines, firstline) corresponding to the
        definition of the controller class. sourcelines is a list of source code
        lines. firstline is the line number of the first source code line."""
        return self.code

    def serialize(self, *args, **kwargs):
        """Returns a serializable object describing this object.

        :return: a serializable dict
        :rtype: dict"""
        kwargs = SardanaBaseObject.serialize(self, *args, **kwargs)
        kwargs['id'] = 0
        kwargs['module'] = self.module_name
        kwargs['file_name'] = self.file_name
        kwargs['file_path'] = self.file_path
        kwargs['path'] = self.path
        kwargs['description'] = self.description
        return kwargs

    def get_brief_description(self, max_chars=60):
        desc = self.description.replace('\n',' ')
        if len(desc) > (max_chars-5):
            desc = desc[:max_chars-5] + '[...]'
        return desc


class SardanaClass(SardanaCode):
    """Object representing a python class."""

    def __init__(self, **kwargs):
        klass = kwargs.pop('klass')
        kwargs['code'] = klass
        kwargs['name'] = kwargs.pop('name', klass.__name__)
        SardanaCode.__init__(self, **kwargs)

    @property
    def klass(self):
        return self.code_object


class SardanaFunction(SardanaCode):
    """Object representing a python function."""

    def __init__(self, **kwargs):
        function = kwargs.pop('function')
        kwargs['code'] = function
        kwargs['name'] = kwargs.pop('name', function.func_name)
        SardanaCode.__init__(self, **kwargs)

    @property
    def function(self):
        return self.code_object




#!/usr/bin/env python
import sys

"""
ControllerTemplate.py: Create a basic controller's template. 
Its parameters are  the file name plus .py, 
    the class inherited if it had (optional)
    and "yes" if you want to use the obsolete convention.
The necessary "defs" are marked as #TODO

python ControllerTemplate.py ExampleClass.py InheritedClass NoCT
"""
__author__ = "Carlos Falcon - cfalcon@cells.es"

class ControllerTemplate():
    def __init__(self,f, e=""):
        self.filename = f
        self.end = e
        self.ind = 'ind' 
    #    pass
    def addHead(self):
        f=open(self.filename,"w")
        f.write('##############################################################################\n'+\
                '##\n'+\
                '## This file is part of Sardana\n'+\
                '##\n'+\
                '## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html\n'+\
                '##\n'+\
                '## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain\n'+\
                '##\n'+\
                '## Sardana is free software: you can redistribute it and/or modify\n'+\
                '## it under the terms of the GNU Lesser General Public License as published by\n'+\
                '## the Free Software Foundation, either version 3 of the License, or\n'+\
                '## (at your option) any later version.\n'+\
                '##\n'+\
                '## Sardana is distributed in the hope that it will be useful,\n'+\
                '## but WITHOUT ANY WARRANTY; without even the implied warranty of\n'+\
                '## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n'+\
                '## GNU Lesser General Public License for more details.\n'+\
                '##\n'+\
                '## You should have received a copy of the GNU Lesser General Public License\n'+\
                '## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.\n'+\
                '##\\n'+\
                '##############################################################################\n\n')\

    def addIncludes(self,inherit,others=None):
        f = open(self.filename,"a")
        text = "from sardana import State\n"
        if inherit!="":
            text = text+"from sardana.pool.controller import "+inherit+"\n"
            if inherit.find("Motor")>=0:
                self.ind = 'axis'
        if others is not None:
            text = text+others
        text = text+"#ADD others includes\n\n"
        f.write(text)
        #f.close()

    def createBasicClass(self):
        f = open(self.filename,"a")
        text = "#TODO - Delete it if you don't need\n"
        text = text +'class BasicClass():\n'+\
               '\tpass\n\n'
        f.write(text)

    def createMainClass(self,inherit):
        f = open(self.filename,"a")
        text = "class "+self.filename[0:len(self.filename)-3]+"("+inherit+"):\n"+\
        '\t"""Description""" #TODO\n'+\
        '\tgender = "Simulation"\n'+\
        '\tmodel  = "Basic"\n'+\
        '\torganization = "CELLS - ALBA"\n'+\
        '\timage = "IMAGE.png"\n'+\
        '\tlogo = "ALBA_logo.png"\n\n'+\
        '\t#TODO - Delete it if you don\'t need\n'+\
        '\tctrl_properties= { \'AAA\' : { \'Type\' : \'DevString\', \'Description\' : \'AAA\' } }\n'+\
        '\taxis_attributes = { \'AAA\' : { \'type\' : str, \'Description\' : \'AAA\' }}\n\n'+\
        '\tMaxDevice = 1024 #TODO Standar value\n\n'

        fun = '# --------------------------------------------------------------------------\n'+\
            '# Init()\n'+\
            '# --------------------------------------------------------------------------\n'+\
            '\tdef __init__(self, inst, props, *args, **kwargs):\n' 
        if inherit!="":
            fun = fun + '\t\t' + inherit + '.__init__(self, inst, props, *args, **kwargs)\n'
        fun = fun + '\t\t#TODO\n'
        text = text + fun
        
        fun = '# --------------------------------------------------------------------------\n'+\
            '# AddDevice/DelDevice()\n'+\
            '# --------------------------------------------------------------------------\n'+\
            '\tdef AddDevice(self,'+self.ind+'):\n'
        fun = fun + '\t\t#TODO\n' 
        fun = fun + '\tdef DeleteDevice(self, '+self.ind+'):\n'
        fun = fun + '\t\t#TODO\n' 
        text = text + fun

        fun = '# --------------------------------------------------------------------------\n'+\
            '# State()\n'+\
            '# --------------------------------------------------------------------------\n'+\
            '\tdef PreStateOne'+self.end+'(self, '+self.ind+'):\n'+'\t\tpass\n' 
        fun = fun + '\tdef StateOne(self, '+self.ind+'):\n'
        fun = fun + '\t\tstate = State.On\n'
        fun = fun + '\t\tstatus = "Undefined"\n'
        if inherit.find("Motor")>=0:
            fun = fun + '\t\tswitchstate = 0"\n'
            fun = fun + '\t\t#TODO\n' 
            fun = fun + '\t\treturn state, status, switchstate"\n'
        else:        
            fun = fun + '\t\t#TODO\n' 
            fun = fun + '\t\treturn state, status\n'

        fun = fun + '\tdef PreStateAll'+self.end+'(self):\n'
        fun = fun + '\t\tpass\n' 

        fun = fun + '\tdef StateAll'+self.end+'(self):\n'
        fun = fun + '\t\tpass\n' 
        text = text + fun

        fun = '# --------------------------------------------------------------------------\n'+\
            '# Read()\n'+\
            '# --------------------------------------------------------------------------\n'+\
            '\tdef PreReadOne'+self.end+'(self, '+self.ind+'):\n'+'\t\tpass\n'
        fun = fun + '\tdef ReadOne(self, '+self.ind+'):\n'
        fun = fun + '\t\t#TODO\n'
        fun = fun + '\tdef PreReadAll'+self.end+'(self):\n'
        fun = fun + '\t\tpass\n' 
        fun = fun + '\tdef ReadAll'+self.end+'(self):\n'
        fun = fun + '\t\tpass\n' 
 
        text = text + fun

        fun = '# --------------------------------------------------------------------------\n'+\
            '# Start/Stop()\n'+\
            '# --------------------------------------------------------------------------\n'+\
            '\tdef PreStartOne'+self.end
        if inherit.find("Motor")>=0:
             fun = fun + '(self, '+self.ind+', pos):\n'
        else:
             fun = fun + '(self, '+self.ind+'):\n'
        fun = fun + '\t\tpass\n' 
        fun = fun + '\tdef StartOne'+self.end+'(self, '+self.ind+', pos):\n'
        fun = fun + '\t\t#TODO\n' 

        fun = fun + '\tdef AbortOne(self, '+self.ind+'):\n'
        fun = fun + '\t\t#TODO\n' 

        fun = fun + '\tdef StopOne(self, '+self.ind+'):\n'
        fun = fun + '\t\tself.AbortOne('+self.ind+')\n' 

        fun = fun + '\tdef PreStartAll'+self.end+'(self):\n'
        fun = fun + '\t\tpass\n' 

        fun = fun + '\tdef StartAll'+self.end+'(self):\n'
        fun = fun + '\t\tpass\n' 

        fun = fun + '\tdef AbortAll(self):\n'
        fun = fun + '\t\tpass\n' 
        text = text + fun

        fun = '# --------------------------------------------------------------------------\n'+\
            '# SetAxisPar/GetAxisPar()\n'+\
            '# --------------------------------------------------------------------------\n'+\
            '\tdef SetAxisPar(self, '+self.ind+', name, value):\n'
        fun = fun + '\t\t#TODO - Delete it if you don\'t need\n' 

        fun = fun + '\tdef GetAxisPar(self, '+self.ind+', name):\n'
        fun = fun + '\t\t#TODO - Delete it if you don\'t need\n' 
        text = text + fun

        fun = '# --------------------------------------------------------------------------\n'+\
            '# SetAxisExtraPar/GetAxisExtraPar()\n'+\
            '# --------------------------------------------------------------------------\n'+\
            '\tdef SetAxisExtraPar(self, '+self.ind+', name, value):\n'
        fun = fun + '\t\t#TODO - Delete it if you don\'t need\n' 

        fun = fun + '\tdef GetAxisExtraPar(self, '+self.ind+', name):\n'
        fun = fun + '\t\t#TODO - Delete it if you don\'t need - \n' 
        text = text + fun
        f.write(text)


def main():
    #Add MACRO_PATH
    filename = ""
    end = ""
    inherit = ""
    if(len(sys.argv) > 1):
        print "Creating " + sys.argv[1]
        filename = sys.argv[1]
        if(len(sys.argv) > 2):
            inherit = sys.argv[2]
        if(len(sys.argv) > 3):
            end = "CT"
        s= ControllerTemplate(filename,end)
        s.addHead()
        s.addIncludes(inherit)
        s.createBasicClass()
        s.createMainClass(inherit)
    else:
        print "Please introduce filename"

if __name__ == "__main__":
    main()



##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This is the standard macro module"""

from __future__ import print_function

__all__ = ["sar_demo"]

import PyTango

from sardana.macroserver.macro import macro, Type

_ENV = "_SAR_DEMO"

def get_free_names(db, prefix, nb, start_at=1):
    ret = []
    i = start_at
    failed = 96
    while len(ret) < nb and failed > 0:
        name = "%s%02d" % (prefix, i)
        try:
            db.get_device_alias(name)
            failed -= 1
        except:
            ret.append(name)
        i += 1
    if len(ret) < nb or failed == 0:
        raise Exception("Too many sardana demos registered on this system.\n"
                        "Please try using a different tango system")
    return ret

@macro()
def clear_sar_demo(self):
    """Undoes changes done with sar_demo"""
    try:
        SAR_DEMO = self.getEnv(_ENV)
    except:
        self.error("No demo has been prepared yet on this sardana!")
        return
    
    self.print("Removing measurement groups...")
    for mg in SAR_DEMO.get("measurement_groups", ()):
        self.udefmeas(mg)
    
    self.print("Removing elements...")
    for elem in SAR_DEMO.get("elements", ()):
        self.udefelem(elem)
    
    self.print("Removing controllers...")
    for ctrl in SAR_DEMO.get("controllers", ()):
        self.udefctrl(ctrl)
    
    self.unsetEnv(_ENV)
    
    self.print("DONE!")
    
@macro()
def sar_demo(self):
    """Sets up a demo environment. It creates many elements for testing"""
    
    try:
        SAR_DEMO = self.getEnv(_ENV)
        self.error("A demo has already been prepared on this sardana")
        return
    except:
        pass
    
    db = PyTango.Database()
    
    mot_ctrl_name = get_free_names(db, "motctrl", 1)[0]
    ct_ctrl_name = get_free_names(db, "ctctrl", 1)[0]
    zerod_ctrl_name = get_free_names(db, "zerodctrl", 1)[0]
    oned_ctrl_name = get_free_names(db, "onedctrl", 1)[0]
    twod_ctrl_name = get_free_names(db, "twodctrl", 1)[0]
    pm_ctrl_name = get_free_names(db, "slitctrl", 1)[0]
    
    motor_names = get_free_names(db, "mot", 4)
    ct_names = get_free_names(db, "ct", 4)
    zerod_names = get_free_names(db, "zerod", 4)
    oned_names = get_free_names(db, "oned", 1)
    twod_names = get_free_names(db, "twod", 1)
    gap, offset = get_free_names(db, "gap", 1) + get_free_names(db, "offset", 1)
    
    mg_name = get_free_names(db, "mntgrp", 1)[0]
    
    pools = self.getPools()
    if not len(pools):
        self.error('This is not a valid sardana demonstration system.\n'
                   'Sardana demonstration systems must be connect to at least '
                   'one Pool')
        return
    pool = pools[0]
    
    self.print("Creating motor controller", mot_ctrl_name, "...")
    self.defctrl("DummyMotorController", mot_ctrl_name)
    for axis, motor_name in enumerate(motor_names, 1):
        self.print("Creating motor", motor_name, "...")
        self.defelem(motor_name , mot_ctrl_name, axis)
        
    self.print("Creating counter controller", ct_ctrl_name, "...")
    self.defctrl("DummyCounterTimerController", ct_ctrl_name)
    for axis, ct_name in enumerate(ct_names, 1):
        self.print("Creating counter channel", ct_name, "...")
        self.defelem(ct_name , ct_ctrl_name, axis)
    
    self.print("Creating 0D controller", zerod_ctrl_name, "...")
    self.defctrl("DummyZeroDController", zerod_ctrl_name)
    for axis, zerod_name in enumerate(zerod_names, 1):
        self.print("Creating 0D channel", zerod_name, "...")
        self.defelem(zerod_name , zerod_ctrl_name, axis)

    self.print("Creating 1D controller", oned_ctrl_name, "...")
    self.defctrl("DummyOneDController", oned_ctrl_name)
    for axis, oned_name in enumerate(oned_names, 1):
        self.print("Creating 1D channel", oned_name, "...")
        self.defelem(oned_name , oned_ctrl_name, axis)

    self.print("Creating 2D controller", twod_ctrl_name, "...")
    self.defctrl("DummyTwoDController", twod_ctrl_name)
    for axis, twod_name in enumerate(twod_names, 1):
        self.print("Creating 2D channel", twod_name, "...")
        self.defelem(twod_name , twod_ctrl_name, axis)
    
    self.print("Creating Slit", pm_ctrl_name, "with", gap, ",", offset, "...")
    sl2t, sl2b = motor_names[:2]
    self.defctrl("Slit", pm_ctrl_name, "sl2t="+sl2t, "sl2b="+sl2b,
                 "Gap="+gap, "Offset="+offset)
   
    self.print("Creating measurement group", mg_name, "...")
    self.defmeas(mg_name, *ct_names)
    
    controllers = pm_ctrl_name, mot_ctrl_name, ct_ctrl_name, \
            zerod_ctrl_name, oned_ctrl_name, twod_ctrl_name
    elements = [gap, offset] + motor_names + ct_names + \
            zerod_names + oned_names + twod_names
    d = dict(controllers=controllers, elements=elements,
             measurement_groups=[mg_name])
    
    self.setEnv(_ENV, d)
    
    self.print("DONE!")

@macro([["motor", Type.Moveable, None, '']])
def mym2(self, pm):
    self.output(pm.getMotorNames())
    elements = map(self.getMoveable, pm.elements)
    self.output(elements)
    self.output(type(pm))
    self.output(type(elements[0]))
 



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

""" """

__docformat__ = 'restructuredtext'

__all__ = ["check_requirements"]

import sys

__requires__ = {
#     module        minimum
    "Python"      : (2,6,0),
    "PyTango"     : (7,2,3),
    "taurus.core" : (3,1,0),
}

def check_requirements(exec_name=None):
    
    if exec_name is None:
        exec_name = sys.argv[0]
    
    pyver_ = __requires__['Python']
    pytangover_ = __requires__['PyTango']
    taurusver_ = __requires__['taurus.core']

    pyver_str_ = ".".join(map(str, pyver_))
    pytangover_str_ = ".".join(map(str, pytangover_))
    taurusver_str_ = ".".join(map(str,taurusver_))
    
    pyver = sys.version_info[:3]
    pyver_str = ".".join(map(str,pyver))

    if pyver < pyver_:
        print "Sardana requires python %s. Installed version is %s" % (pyver_str_, pyver_str)
        sys.exit(-1)
    
    pytangover = None
    try:
        import PyTango
        pytangover = PyTango.Release.version_info[:3]
    except ImportError:
        pass
    except:
        pytangover = tuple(map(int, PyTango.__version__.split('.', 3)))

    if pytangover is None:
        print "%s requires PyTango %s. No version installed" % (exec_name, pytangover_str_,)
        sys.exit(-1)
    if pytangover < pytangover_:
        pytangover_str = ".".join(map(str,pytangover))
        print "%s requires PyTango %s. Installed version is %s" % (exec_name, pytangover_str_, pytangover_str)
        sys.exit(-1)
    
    taurusver = None
    try:
        import taurus
        taurusver = taurus.Release.version_info[:3]
    except ImportError:
        pass
    except:
        taurusver = tuple(map(int, taurus.Release.version.split('.', 3)))

    if taurusver is None:
        print "%s requires taurus %s. No version installed" % (exec_name, taurusver_str_,)
        sys.exit(-1)
    if taurusver < taurusver_:
        taurusver_str = ".".join(map(str,taurusver))
        print "%s requires taurus %s. Installed version is %s" % (exec_name, taurusver_str_, taurusver_str)
        sys.exit(-1)
    
    try:
        from lxml import etree
    except:
        print "Could not find any suitable XML library"
        sys.exit(-1)
